﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;
using System.Threading;

namespace AdEmulator
{
    public class Emulador
    {
        private static Emulador emulador = null;
        SerialPort porta;
        String buffer;
        List<String> comandos;
        public double horaInicial = DateTime.Now.TimeOfDay.TotalMilliseconds;
        private int ASF { get; set; }
        public string curvaAutomatica = "SENOIDAL";
        private long points;
        private System.Timers.Timer pointsTimer;
        public Boolean Automatico { get; set; }
        public int Pontos { get; set; }

        public event EventHandler ComandosChanged;

        public static Emulador getInstance()
        {
            if (emulador == null)
            {
                emulador = new Emulador();
            }
            return emulador;
        }

        private Emulador()
        {
            this.buffer = "";
            this.comandos = new List<string>();
            this.ASF = 0;
            this.points = 0;
            this.pointsTimer = new System.Timers.Timer(5);
            this.pointsTimer.Elapsed += new System.Timers.ElapsedEventHandler(pointsTimer_Elapsed);
            this.pointsTimer.Start();
            this.ComandosChanged += new EventHandler(Emulador_ComandosChanged);
            this.Automatico = false;
            this.Pontos = 0;
        }

        void pointsTimer_Elapsed(object sender, System.Timers.ElapsedEventArgs e)
        { 
            lock (this.pointsTimer)
            {
                if (Automatico)
                {
                    double milisec = e.SignalTime.TimeOfDay.TotalMilliseconds - horaInicial;
                    if (this.curvaAutomatica == "SENOIDAL")
                    {
                        this.points = (long)((Math.Sin(Math.PI * 2 * milisec / 120000) + 1) * 1000 / 2);
                    }
                    else
                    {
                        this.points = (long)((milisec / 120000) * 1000);
                    }
                }
                else
                {
                    this.points = Pontos;
                }
            }
        }

        void Emulador_ComandosChanged(object sender, EventArgs e)
        {
            foreach (String c in this.readComandos())
            {
                enviaResposta(c);
            }
        }

        public Boolean estaConectado()
        {
            if (porta == null)
            {
                return false;
            }
            return porta.IsOpen;
        }

        public void conectar(String nomePorta, int baudrate, Parity paridade)
        {
            porta = new SerialPort(nomePorta, baudrate, paridade);
            porta.DataReceived += new SerialDataReceivedEventHandler(porta_DataReceived);
            porta.Open();
        }

        protected virtual void OnComandosChanged(EventArgs e)
        {
            if (ComandosChanged != null)
                ComandosChanged(this, e);
        }

        private void porta_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            String received = porta.ReadExisting();
            buffer += received;
            if (buffer.Contains(";"))
            {
                String[] arrayComandos = buffer.Split(';');
                comandos.Clear();
                for (int i = 0; i < arrayComandos.Length-1; i++)
                {
                    comandos.Add(arrayComandos[i]);
                }
                buffer = arrayComandos[arrayComandos.Length - 1];
                OnComandosChanged(EventArgs.Empty);
            }
        }

        public void desconectar()
        {
            porta.Close();
        }

        private void enviaResposta(String comando)
        {
            if (comando.ToUpper().Equals("ASF?"))
            {
                Thread.Sleep(30);
                porta.Write(this.ASF.ToString("00") + "\r\n");
            }
            else if (comando.ToUpper().StartsWith("ASF"))
            {
                try
                {
                    int asf = int.Parse(comando.Substring(3));
                    this.ASF = asf;
                    Thread.Sleep(30);
                    porta.Write("0" + "\r\n");
                }
                catch
                {
                    Thread.Sleep(30);
                    porta.Write("?" + "\r\n");
                }
            }
            else if (comando.ToUpper().StartsWith("MSV?"))
            {
                try
                {
                    int msv = int.Parse(comando.Substring(4));
                    for (int i = 0; i < msv; i++)
                    {
                        Thread.Sleep(30);
                        lock (this.pointsTimer)
                        {
                            porta.Write(this.points.ToString("00000000") + "\r\n");
                        }
                    }
                }
                catch
                {
                    Thread.Sleep(30);
                    porta.Write("?" + "\r\n");
                }
            }
            else
            {
                Thread.Sleep(30);
                porta.Write("?" + "\r\n");
            }
        }

        public List<String> readComandos()
        {
            List<String> c = new List<String>();
            c.AddRange(comandos);
            return c;
        }
    }
}
