﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;
using TestView.Ensaios.Utilidades.CorposDeProva;

namespace ModuloEnsaioTracao.Ensaios.CorposDeProva
{
    [DataContract]
    public abstract class CorpoDeProvaTracao : CorpoDeProva
    {
        #region Construtor e membros

        private Double _comprimentoOriginal;
        private Double _comprimentoFinal;

        [DataMember]
        public Double ComprimentoOriginal
        {
            get
            {
                return this._comprimentoOriginal;
            }
            set
            {
                this._comprimentoOriginal = value;
                this.informCorpoDeProvaListeners();
            }
        }
        [DataMember]
        public Double ComprimentoFinal
        {
            get
            {
                return this._comprimentoFinal;
            }
            set 
            {
                this._comprimentoFinal = value;
                this.informCorpoDeProvaListeners();
            }
        }

        #endregion
    }

    [DataContract]
    public class Quadrado : CorpoDeProvaTracao
    {
        #region Construtor e membros

        private double _lado;
        private static String _nome = "Quadrado";

        public Quadrado(double lado)
        {
            this.Lado = lado;
        }

        [DataMember]
        public double Lado
        {
            get
            {
                return this._lado;
            }
            set
            {
                this._lado = value;
                this.informCorpoDeProvaListeners();
            }
        }

        #endregion

        public override object Clone()
        {
            Quadrado quadrado = new Quadrado(this.Lado);
            quadrado.ComprimentoFinal = this.ComprimentoFinal;
            quadrado.ComprimentoOriginal = this.ComprimentoOriginal;
            return quadrado;
        }
        public override double AreaEquivalente
        {
            get
            {
                return this.Lado * this.Lado;
            }
        }
    }

    [DataContract]
    public class Plano : CorpoDeProvaTracao
    {
        #region Construtor e membros

        private double _base;
        private double _altura;
        private double _alturaDeformacaoInferior;
        private double _alturaDeformacaoCentral;
        private double _alturaDeformacaoSuperior;
        private double _baseDeformacao;
        private static String _nome = "Plano";

        public Plano(double bse, double altura)
        {
            this._base = bse;
            this._altura = altura;
        }

        [DataMember]
        public double Base
        {
            get
            {
                return this._base;
            }
            set
            {
                this._base = value;
                this.informCorpoDeProvaListeners();
            }
        }
        [DataMember]
        public double Altura
        {
            get
            {
                return this._altura;
            }
            set
            {
                this._altura = value;
                this.informCorpoDeProvaListeners();
            }
        }
        [DataMember]
        public double AlturaDeformacaoInferior
        {
            get
            {
                return this._alturaDeformacaoInferior;
            }
            set
            {
                this._alturaDeformacaoInferior = value;
                this.informCorpoDeProvaListeners();
            }
        }
        [DataMember]
        public double AlturaDeformacaoCentral
        {
            get
            {
                return this._alturaDeformacaoCentral;
            }
            set
            {
                this._alturaDeformacaoCentral = value;
                this.informCorpoDeProvaListeners();
            }
        }
        [DataMember]
        public double AlturaDeformacaoSuperior
        {
            get
            {
                return this._alturaDeformacaoSuperior;
            }
            set
            {
                this._alturaDeformacaoSuperior = value;
                this.informCorpoDeProvaListeners();
            }
        }
        [DataMember]
        public double BaseDeformacao
        {
            get
            {
                return this._baseDeformacao;
            }
            set
            {
                this._baseDeformacao = value;
                this.informCorpoDeProvaListeners();
            }
        }
        #endregion

        public override object Clone()
        {
            Plano plano = new Plano(this.Base, this.Altura);
            plano.AlturaDeformacaoCentral = this.AlturaDeformacaoCentral;
            plano.AlturaDeformacaoInferior = this.AlturaDeformacaoInferior;
            plano.AlturaDeformacaoSuperior = this.AlturaDeformacaoSuperior;
            plano.BaseDeformacao = this.BaseDeformacao;
            plano.ComprimentoFinal = this.ComprimentoFinal;
            plano.ComprimentoOriginal = this.ComprimentoOriginal;
            return plano;
        }
        public override double AreaEquivalente
        {
            get
            {
                return this.Base * this.Altura;
            }
        }
        public double AreaFinal
        {
            get
            {
                if (!double.IsNaN(this.AlturaDeformacaoCentral) && !double.IsNaN(this.AlturaDeformacaoInferior) && !double.IsNaN(this.AlturaDeformacaoSuperior) && !double.IsNaN(this.BaseDeformacao))
                {
                    return (this.AlturaDeformacaoInferior + 4 * this.AlturaDeformacaoCentral + this.AlturaDeformacaoSuperior) * this.BaseDeformacao / 6;
                }
                else
                {
                    return Double.NaN;
                }
            }
        }
        public Double ReducaoPercentalDeArea
        {
            get
            {
                if (!double.IsNaN(this.AreaFinal) && !double.IsNaN(this.AreaEquivalente))
                {
                    return (this.AreaEquivalente - this.AreaFinal) * 100 / this.AreaEquivalente;
                }
                else
                {
                    return double.NaN;
                }
            }
        }
    }

    [DataContract]
    public class Cilindrico : CorpoDeProvaTracao
    {
        #region Construtor e membros

        private double _diametroOriginal;
        private double _diametroFinal;

        public Cilindrico(double diametro)
        {
            this._diametroOriginal = diametro;
        }

        [DataMember]
        public double DiametroOriginal
        {
            get
            {
                return this._diametroOriginal;
            }
            set
            {
                this._diametroOriginal = value;
                this.informCorpoDeProvaListeners();
            }
        }
        [DataMember]
        public double DiametroFinal
        {
            get
            {
                return this._diametroFinal;
            }
            set
            {
                this._diametroFinal = value;
                this.informCorpoDeProvaListeners();
            }
        }

        #endregion
        public override object Clone()
        {
            Cilindrico cilindrico = new Cilindrico(this.DiametroOriginal);
            cilindrico.ComprimentoFinal = this.ComprimentoFinal;
            cilindrico.ComprimentoOriginal = this.ComprimentoOriginal;
            cilindrico.DiametroFinal = this.DiametroFinal;
            return cilindrico;
        }
        public override double AreaEquivalente
        {
            get
            {
                return Math.PI * this.DiametroOriginal * this.DiametroOriginal / 4;
            }
        }
        public Double AreaFinal
        {
            get
            {
                if (!double.IsNaN(this.DiametroFinal))
                {
                    return Math.PI * this.DiametroFinal * this.DiametroFinal / 4;
                }
                else
                {
                    return double.NaN;
                }
            }
        }
        public Double ReducaoPercentalDeArea
        {
            get
            {
                if (!double.IsNaN(this.AreaFinal) && !double.IsNaN(this.AreaEquivalente))
                {
                    return (this.AreaEquivalente - this.AreaFinal) * 100 / this.AreaEquivalente;
                }
                else
                {
                    return double.NaN;
                }
            }
        }
    }
}
