﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TestView.Ensaios;
using TestView.Ensaios.Utilidades.CorposDeProva;
using TestView.Condicionador;
using TestView.Ensaios.Utilidades;
using ModuloEnsaioTracao.Graficos;
using TestView.Serializacao;
using TestView.Ensaios.Utilidades.Coleta;
using System.Runtime.Serialization;
using System.IO;
using ModuloEnsaioTracao.Ensaios.CorposDeProva;
using ModuloEnsaioTracao.Ensaios;
using TestView.MyMath;
using TestView.Ensaios.Utilidades.Instrumentacao;
using System.Drawing;
using TestView.Relatorios;

namespace ModuloEnsaioTracao.Ensaios
{
    [DataContract]
    [KnownType(typeof(Quadrado))]
    [KnownType(typeof(Cilindrico))]
    [KnownType(typeof(Plano))]
    public class EnsaioTracao : Ensaio
    {
        #region Construtor e membros

        private String _id;
        private DateTime _dataEnsaio;
        private EstadoEnsaio _estadoEnsaio;
        private CorpoDeProva _corpoProva;
        private Aquisicao _aquisicao;
        private DateTime _tempoInicioEnsaio;
        private GraficoEnsaioTracao _grafico;
        private OpcoesEnsaioTracao _opcoesEnsaio;
        private List<InformacaoAdicional> _informacoesAdicionais;
        private Double _forcaMaxima;
        private Double _velocidadeInstantaneaForca;
        private Double _velocidadeInstantaneaDeslocamento;
        private Double _moduloDeElasticidade;
        private Double _interseccaoOrdenadaModuloElasticidade;
        private Unidade _unidadeModuloElasticidade;
        private Unidade _unidadeInterseccaoOrdenadaModuloElasticidade;
        private Boolean _extensometroRemovido;
        private Boolean _retaParalelaAtingida;
        private Double _tensaoEscoamento;
        private RelatorioEnsaioTracao _relatorio;

        public EnsaioTracao()
        {
            this._id = Guid.NewGuid().ToString();
            this._estadoEnsaio = EstadoEnsaio.NAO_INICIADO;
            this._dataEnsaio = DateTime.Now;
            this._corpoProva = new Cilindrico(1);
            this._aquisicao = new Aquisicao();
            this._forcaMaxima = double.NaN;
            this._velocidadeInstantaneaForca = double.NaN;
            this._velocidadeInstantaneaDeslocamento = double.NaN;
            this._moduloDeElasticidade = double.NaN;
            this._interseccaoOrdenadaModuloElasticidade = double.NaN;
            this._unidadeInterseccaoOrdenadaModuloElasticidade = Unidade.MPA;
            this._unidadeModuloElasticidade = Unidade.MPA;
            this._informacoesAdicionais = new List<InformacaoAdicional>();
            this._grafico = new GraficoEnsaioTracao();
            this._opcoesEnsaio = new OpcoesEnsaioTracao();
            this._extensometroRemovido = false;
            this._retaParalelaAtingida = false;
            this._tensaoEscoamento = double.NaN;
            this._relatorio = new RelatorioEnsaioTracao();
        }

        [DataMember]
        public override string  Id
        {
	          get 
	        {
                return this._id;
	        }
	          set 
	        {
                this._id = value;
	        }
        }
        [DataMember]
        public override EstadoEnsaio EstadoEnsaio
        {
            get
            {
                return this._estadoEnsaio;
            }
            set
            {
                this._estadoEnsaio = value;
                this.informEstadoEnsaioListeners();
            }
        }
        [DataMember]
        public override DateTime DataEnsaio
        {
            get
            {
                return this._dataEnsaio;
            }
            set
            {
                this._dataEnsaio = value;
            }
        }
        [DataMember]
        public override CorpoDeProva CorpoDeProva
        {
            get
            {
                return this._corpoProva;
            }
            set
            {
                this._corpoProva = value;
                this.informCorpoDeProvaListeners();
            }
        }
        [DataMember]
        public override Aquisicao Aquisicao
        {
            get
            {
                return this._aquisicao;
            }
            set
            {
                this._aquisicao = value;
            }
        }
        [DataMember]
        public DateTime HoraInicioEnsaio
        {
            get
            {
                return this._tempoInicioEnsaio;
            }
            set
            {
                this._tempoInicioEnsaio = value;
            }
        }
        [DataMember]
        public List<InformacaoAdicional> InformacoesAdicionais
        {
            get
            {
                return this._informacoesAdicionais;
            }
            set
            {
                this._informacoesAdicionais = value;
                this.informInformacoesAdicionaisListeners();
            }
        }
        [DataMember]
        public GraficoEnsaioTracao Grafico
        {
            get
            {
                return this._grafico;
            }
            set
            {
                this._grafico = value;
            }
        }
        [DataMember]
        public OpcoesEnsaioTracao OpcoesDeEnsaio
        {
            get
            {
                return this._opcoesEnsaio;
            }
            set
            {
                this._opcoesEnsaio = value;
            }
        }
        public override Relatorio Relatorio
        {
            get
            {
                return this._relatorio;
            }
        }
        [DataMember]
        public RelatorioEnsaioTracao RelatorioEnsaioTracao
        {
            get
            {
                return this._relatorio;
            }
            set
            {
                this._relatorio = value;
            }
        }
        [DataMember]
        public Boolean ExtensometroRemovido
        {
            get
            {
                return this._extensometroRemovido;
            }
            set
            {
                this._extensometroRemovido = value;
                this.informExtensometroRemovidoListeners();
            }
        }
        [DataMember]
        public Boolean RetaParalelaModuloAtingida
        {
            get
            {
                return this._retaParalelaAtingida;
            }
            set
            {
                this._retaParalelaAtingida = value;
            }
        }
        public SerieColeta SerieForca
        {
            get
            {
                foreach (SerieColeta serie in this.Aquisicao.SeriesTemporais)
                {
                    if (serie.Calibracao.TipoTransdutor.Equals(Transdutor.FORCA))
                    {
                        return serie;
                    }
                }
                return null;
            }
        }
        public SerieColeta SerieDeformacao
        {
            get
            {
                foreach (SerieColeta serie in this.Aquisicao.SeriesTemporais)
                {
                    if (serie.Calibracao.TipoTransdutor.Equals(Transdutor.DEFORMACAO))
                    {
                        return serie;
                    }
                }
                return null;
            }
        }
        public SerieColeta SerieDeslocamento
        {
            get
            {
                foreach (SerieColeta serie in this.Aquisicao.SeriesTemporais)
                {
                    if (serie.Calibracao.TipoTransdutor.Equals(Transdutor.DESLOCAMENTO))
                    {
                        return serie;
                    }
                }
                return null;
            }
        }
        public Extensometro Extensometro
        {
            get
            {
                if (this.SerieDeformacao != null)
                {
                    return (Extensometro)this.SerieDeformacao.Instrumento;
                }
                else
                {
                    return null;
                }
            }
        }

        #region Dados do ensaio

        [DataMember]
        public double TensaoEscoamento
        {
            get
            {
                return this._tensaoEscoamento;
            }
            set
            {
                this._tensaoEscoamento = value;
                this.informTensaoEscoamentoListeners();
            }
        }
        public Unidade UnidadeTensaoEscoamento
        {
            get
            {
                return Unidade.MPA;
            }
        }
        public double ForcaEscoamento
        {
            get
            {
                if (double.IsNaN(this._tensaoEscoamento))
                {
                    return double.NaN;
                }
                else
                {
                    double forcaEscoamentoNewton = Conversor.converterTensao(this.TensaoEscoamento, this.UnidadeTensaoEscoamento, Unidade.MPA) / this.CorpoDeProva.AreaEquivalente;
                    return Conversor.converterForca(forcaEscoamentoNewton, Unidade.N, this.UnidadeForcaEscoamento);
                }
            }
            set
            {
                this._tensaoEscoamento = value;
                this.informForcaEscoamentoListeners();
            }
        }
        public Unidade UnidadeForcaEscoamento
        {
            get
            {
                return Unidade.KGF;
            }
        }
        [DataMember]
        public double ForcaMaxima
        {
            get
            {
                return this._forcaMaxima;
            }
            set
            {
                this._forcaMaxima = value;
                this.informForcaMaximaListeners();
                this.informTensaoMaximaListeners();
            }
        }
        public Unidade UnidadeForcaMaxima
        {
            get
            {
                return this.SerieForca.Calibracao.Unidade;
            }
        }
        public double TensaoMaxima
        {
            get
            {
                if (!double.IsNaN(this.ForcaMaxima))
                {
                    return Conversor.converterForca(this._forcaMaxima, this.SerieForca.Calibracao.Unidade, Unidade.N) / this.CorpoDeProva.AreaEquivalente;
                }
                else
                {
                    return double.NaN;
                }
            }
        }
        public Unidade UnidadeTensaoMaxima
        {
            get
            {
                return Unidade.MPA;
            }
        }
        public double VelocidadeInstantaneaForca
        {
            get
            {
                return this._velocidadeInstantaneaForca;
            }
            set
            {
                this._velocidadeInstantaneaForca = value;
                this.informVelocidadeForcaListeners();
                this.informVelocidadeTensaoListeners();
            }
        }
        public Unidade UnidadeVelocidadeForca
        {
            get
            {
                return Unidade.KGF_S;
            }
        }
        public double VelocidadeInstantaneaTensao
        {
            get
            {
                if (double.IsNaN(this.VelocidadeInstantaneaForca))
                {
                    return double.NaN;
                }
                else
                {
                    return Conversor.converterForca(this.VelocidadeInstantaneaForca, this.SerieForca.Calibracao.Unidade, Unidade.N) / this.CorpoDeProva.AreaEquivalente;
                }
            }
        }
        public Unidade UnidadeVelocidadeTensao
        {
            get
            {
                return Unidade.MPA_S;
            }
        }
        public double VelocidadeInstantaneaDeslocamento
        {
            get
            {
                return this._velocidadeInstantaneaDeslocamento;
            }
            set
            {
                this._velocidadeInstantaneaDeslocamento = value;
                this.informVelocidadeDeslocamentoListeners();
            }
        }
        public Unidade UnidadeVelocidadeDeslocamento
        {
            get
            {
                return Unidade.MM_S;
            }
        }
        [DataMember]
        public double ModuloDeElasticidade
        {
            get
            {
                return this._moduloDeElasticidade;
            }
            set
            {
                this._moduloDeElasticidade = value;
                this.informModuloElasticidadeListeners();
            }
        }
        [DataMember]
        public double InterseccaoOrdenadaModuloDeElasticidade
        {
            get
            {
                return this._interseccaoOrdenadaModuloElasticidade;
            }
            set
            {
                this._interseccaoOrdenadaModuloElasticidade = value;
            }
        }
        public Unidade UnidadeModuloElasticidade
        {
            get
            {
                return Unidade.MPA;
            }
        }
        public Unidade UnidadeInterseccaoOrdenadaModuloDeElasticidade
        {
            get
            {
                return Unidade.MPA;
            }
        }

        #endregion

        #endregion

        #region Serializacao

        public override void serializaEnsaio(Stream stream)
        {
            Serializador.serializaObjeto(stream, this, typeof(EnsaioTracao));
        }

        public override Ensaio deserializaEnsaio(Stream stream)
        {
            Ensaio ensaio = (Ensaio)Serializador.deserializaObjeto(stream, typeof(EnsaioTracao));
            return ensaio;
        }

        #endregion

        #region Misc

        public override string ToString()
        {
            return String.Format("Ensaio de Tração {0:dd-MM-yy HH:mm}", this.DataEnsaio);
        }

        #endregion

        #region Eventos

        #region Força Máxima

        public delegate void ForcaMaximaChangedEvent(object sender, ForcaMaximaChangedEventArgs e);

        public event ForcaMaximaChangedEvent ForcaMaximaChanged;

        public class ForcaMaximaChangedEventArgs : EventArgs
        {
            public ForcaMaximaChangedEventArgs(Double forcaMaxima)
            {
                this.ForcaMaxima = forcaMaxima;
            }

            public Double ForcaMaxima
            {
                get;
                set;
            }
        }

        protected void informForcaMaximaListeners()
        {
            if (ForcaMaximaChanged != null)
            {
                ForcaMaximaChanged(this, new ForcaMaximaChangedEventArgs(this.ForcaMaxima));
            }
        }

        #endregion

        #region Tensão Máxima

        public delegate void TensaoMaximaChangedEvent(object sender, TensaoMaximaChangedEventArgs e);

        public event TensaoMaximaChangedEvent TensaoMaximaChanged;

        public class TensaoMaximaChangedEventArgs : EventArgs
        {
            public TensaoMaximaChangedEventArgs(Double tensaoMaxima)
            {
                this.TensaoMaxima = tensaoMaxima;
            }

            public Double TensaoMaxima
            {
                get;
                set;
            }
        }

        protected void informTensaoMaximaListeners()
        {
            if (TensaoMaximaChanged != null)
            {
                TensaoMaximaChanged(this, new TensaoMaximaChangedEventArgs(this.TensaoMaxima));
            }
        }

        #endregion

        #region Modulo Elasticidade

        public delegate void ModuloElasticidadeChangedEvent(object sender, ModuloElasticidadeChangedEventArgs e);

        public event ModuloElasticidadeChangedEvent ModuloElasticidadeChanged;

        public class ModuloElasticidadeChangedEventArgs : EventArgs
        {
            public ModuloElasticidadeChangedEventArgs(Double modulo)
            {
                this.ModuloElasticidade = modulo;
            }

            public Double ModuloElasticidade
            {
                get;
                set;
            }
        }

        protected void informModuloElasticidadeListeners()
        {
            if (ModuloElasticidadeChanged != null)
            {
                ModuloElasticidadeChanged(this, new ModuloElasticidadeChangedEventArgs(this.ModuloDeElasticidade));
            }
        }

        #endregion

        #region Tensão Escoamento

        public delegate void TensaoEscoamentoChangedEvent(object sender, TensaoEscoamentoChangedEventArgs e);

        public event TensaoEscoamentoChangedEvent TensaoEscoamentoChanged;

        public class TensaoEscoamentoChangedEventArgs : EventArgs
        {
            public TensaoEscoamentoChangedEventArgs(Double tensaoEscoamento)
            {
                this.TensaoEscoamento = tensaoEscoamento;
            }

            public Double TensaoEscoamento
            {
                get;
                set;
            }
        }

        protected void informTensaoEscoamentoListeners()
        {
            if (TensaoEscoamentoChanged != null)
            {
                TensaoEscoamentoChanged(this, new TensaoEscoamentoChangedEventArgs(this.TensaoEscoamento));
            }
        }

        #endregion

        #region Força Escoamento

        public delegate void ForcaEscoamentoChangedEvent(object sender, ForcaEscoamentoChangedEventArgs e);

        public event ForcaEscoamentoChangedEvent ForcaEscoamentoChanged;

        public class ForcaEscoamentoChangedEventArgs : EventArgs
        {
            public ForcaEscoamentoChangedEventArgs(Double forcaEscoamento)
            {
                this.ForcaEscoamento = forcaEscoamento;
            }

            public Double ForcaEscoamento
            {
                get;
                set;
            }
        }

        protected void informForcaEscoamentoListeners()
        {
            if (ForcaEscoamentoChanged != null)
            {
                ForcaEscoamentoChanged(this, new ForcaEscoamentoChangedEventArgs(this.ForcaEscoamento));
            }
        }

        #endregion

        #region Velocidade Força

        public delegate void VelocidadeForcaChangedEvent(object sender, VelocidadeForcaChangedEventArgs e);

        public event VelocidadeForcaChangedEvent VelocidadeForcaChanged;

        public class VelocidadeForcaChangedEventArgs : EventArgs
        {
            public VelocidadeForcaChangedEventArgs(Double velocidadeForca)
            {
                this.VelocidadeForca = velocidadeForca;
            }

            public Double VelocidadeForca
            {
                get;
                set;
            }
        }

        protected void informVelocidadeForcaListeners()
        {
            if (VelocidadeForcaChanged != null)
            {
                VelocidadeForcaChanged(this, new VelocidadeForcaChangedEventArgs(this.VelocidadeInstantaneaForca));
            }
        }

        #endregion

        #region Velocidade Tensão

        public delegate void VelocidadeTensaoChangedEvent(object sender, VelocidadeTensaoChangedEventArgs e);

        public event VelocidadeTensaoChangedEvent VelocidadeTensaoChanged;

        public class VelocidadeTensaoChangedEventArgs : EventArgs
        {
            public VelocidadeTensaoChangedEventArgs(Double velocidadeTensao)
            {
                this.VelocidadeTensao = velocidadeTensao;
            }

            public Double VelocidadeTensao
            {
                get;
                set;
            }
        }

        protected void informVelocidadeTensaoListeners()
        {
            if (VelocidadeTensaoChanged != null)
            {
                VelocidadeTensaoChanged(this, new VelocidadeTensaoChangedEventArgs(this.VelocidadeInstantaneaTensao));
            }
        }

        #endregion

        #region Velocidade Deslocamento

        public delegate void VelocidadeDeslocamentoChangedEvent(object sender, VelocidadeDeslocamentoChangedEventArgs e);

        public event VelocidadeDeslocamentoChangedEvent VelocidadeDeslocamentoChanged;

        public class VelocidadeDeslocamentoChangedEventArgs : EventArgs
        {
            public VelocidadeDeslocamentoChangedEventArgs(Double velocidadeDeslocamento)
            {
                this.VelocidadeDeslocamento = velocidadeDeslocamento;
            }

            public Double VelocidadeDeslocamento
            {
                get;
                set;
            }
        }

        protected void informVelocidadeDeslocamentoListeners()
        {
            if (VelocidadeDeslocamentoChanged != null)
            {
                VelocidadeDeslocamentoChanged(this, new VelocidadeDeslocamentoChangedEventArgs(this.VelocidadeInstantaneaDeslocamento));
            }
        }

        #endregion

        #region Corpo de prova

        public delegate void CorpoDeProvaChangedEvent(object sender, CorpoDeProvaChangedEventArgs e);

        public event CorpoDeProvaChangedEvent CorpoDeProvaChanged;

        public class CorpoDeProvaChangedEventArgs : EventArgs
        {
            public CorpoDeProvaChangedEventArgs(CorpoDeProva corpoProva)
            {
                this.CorpoDeProva = corpoProva;
            }

            public CorpoDeProva CorpoDeProva
            {
                get;
                set;
            }
        }

        protected void informCorpoDeProvaListeners()
        {
            if (CorpoDeProvaChanged != null)
            {
                CorpoDeProvaChanged(this, new CorpoDeProvaChangedEventArgs(this.CorpoDeProva));
            }
        }

        #endregion

        #region Informações Adicionais

        public delegate void InformacoesAdicionaisChangedEvent(object sender, InformacoesAdicionaisChangedEventArgs e);

        public event InformacoesAdicionaisChangedEvent InformacoesAdicionaisChanged;

        public class InformacoesAdicionaisChangedEventArgs : EventArgs
        {
            public InformacoesAdicionaisChangedEventArgs(List<InformacaoAdicional> informacoes)
            {
                this.InformacoesAdicionais = informacoes;
            }

            public List<InformacaoAdicional> InformacoesAdicionais
            {
                get;
                set;
            }
        }

        protected void informInformacoesAdicionaisListeners()
        {
            if (InformacoesAdicionaisChanged != null)
            {
                InformacoesAdicionaisChanged(this, new InformacoesAdicionaisChangedEventArgs(this.InformacoesAdicionais));
            }
        }

        #endregion

        #region Extensometro Removido

        public delegate void ExtensometroRemovidoEvent(object sender, ExtensometroRemovidoEventArgs e);

        public event ExtensometroRemovidoEvent ExtensometroRemovidoChanged;

        public class ExtensometroRemovidoEventArgs : EventArgs
        {
            public ExtensometroRemovidoEventArgs(bool removido)
            {
                this.ExtensometroRemovido = removido;
            }

            public Boolean ExtensometroRemovido
            {
                get;
                set;
            }
        }

        protected void informExtensometroRemovidoListeners()
        {
            if (ExtensometroRemovidoChanged != null)
            {
                ExtensometroRemovidoChanged(this, new ExtensometroRemovidoEventArgs(this.ExtensometroRemovido));
            }
        }

        #endregion

        #endregion
    }
}
