﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;

namespace ModuloEnsaioTracao.Ensaios
{
    [DataContract]
    public class OpcoesRelatorio : ICloneable
    {
        #region Construtor e membros

        private Boolean _informacoesAdicionais;
        private Boolean _forcaMaxima;
        private Boolean _tensaoMaxima;
        private Boolean _moduloElasticidade;
        private Boolean _tensaoEscoamento;
        private Boolean _forcaEscoamento;
        private Boolean _areaInicial;
        private String _informacoesEmpresa;
        private String _caminhoLogotipo;
        private Boolean _logotipo;

        public OpcoesRelatorio()
        {
            this.InformacoesAdicionais = true;
            this.ForcaMaxima = true;
            this.TensaoMaxima = true;
            this.ModuloElasticidade = true;
            this.TensaoEscoamento = true;
            this.ForcaEscoamento = true;
            this.AreaInicial = true;
            this.CaminhoLogotipo = String.Empty;
            this.Logotipo = false;
        }

        [DataMember]
        public Boolean InformacoesAdicionais
        {
            get
            {
                return this._informacoesAdicionais;
            }
            set
            {
                this._informacoesAdicionais = value;
            }
        }
        [DataMember]
        public Boolean ForcaMaxima
        {
            get
            {
                return this._forcaMaxima;
            }
            set
            {
                this._forcaMaxima = value;
            }
        }
        [DataMember]
        public Boolean TensaoMaxima
        {
            get
            {
                return this._tensaoMaxima;
            }
            set
            {
                this._tensaoMaxima = value;
            }
        }
        [DataMember]
        public Boolean ModuloElasticidade
        {
            get
            {
                return this._moduloElasticidade;
            }
            set
            {
                this._moduloElasticidade = value;
            }
        }
        [DataMember]
        public Boolean TensaoEscoamento
        {
            get
            {
                return this._tensaoEscoamento;
            }
            set
            {
                this._tensaoEscoamento = value;
            }
        }
        [DataMember]
        public Boolean ForcaEscoamento
        {
            get
            {
                return this._forcaEscoamento;
            }
            set
            {
                this._forcaEscoamento = value;
            }
        }
        [DataMember]
        public Boolean AreaInicial
        {
            get
            {
                return this._areaInicial;
            }
            set
            {
                this._areaInicial = value;
            }
        }
        [DataMember]
        public String InformacoesEmpresa
        {
            get
            {
                return this._informacoesEmpresa;
            }
            set
            {
                this._informacoesEmpresa = value;
            }
        }
        [DataMember]
        public String CaminhoLogotipo
        {
            get
            {
                return this._caminhoLogotipo;
            }
            set
            {
                this._caminhoLogotipo = value;
                this.informCaminhoLogotipoListeners();
            }
        }
        [DataMember]
        public Boolean Logotipo
        {
            get
            {
                return this._logotipo;
            }
            set
            {
                this._logotipo = value;
            }
        }

        #endregion

        #region Salvar opcoes

        public void salvaOpcoesRelatorio(OpcoesRelatorio opcoes)
        {
            this.InformacoesAdicionais = opcoes.InformacoesAdicionais;
            this.ForcaMaxima = opcoes.ForcaMaxima;
            this.TensaoMaxima = opcoes.TensaoMaxima;
            this.ModuloElasticidade = opcoes.ModuloElasticidade;
            this.TensaoEscoamento = opcoes.TensaoEscoamento;
            this.ForcaEscoamento = opcoes.ForcaEscoamento;
            this.AreaInicial = opcoes.AreaInicial;
            this.InformacoesEmpresa = opcoes.InformacoesEmpresa;
            this.CaminhoLogotipo = opcoes.CaminhoLogotipo;
            this.Logotipo = opcoes.Logotipo;
            this.informOpcoesDoRelatorioListeners();
        }

        #endregion

        #region Eventos

        #region OpcoesDoRelatorio

        public delegate void OpcoesDoRelatorioChangedEvent(object sender, OpcoesDoRelatorioChangedEventArgs e);

        public event OpcoesDoRelatorioChangedEvent OpcoesDoRelatorioChanged;

        public class OpcoesDoRelatorioChangedEventArgs : EventArgs
        {
            public OpcoesDoRelatorioChangedEventArgs(OpcoesRelatorio opcoes)
            {
                this.OpcoesRelatorio = opcoes;
            }

            public OpcoesRelatorio OpcoesRelatorio
            {
                get;
                set;
            }
        }

        private void informOpcoesDoRelatorioListeners()
        {
            if (OpcoesDoRelatorioChanged != null)
            {
                OpcoesDoRelatorioChanged(this, new OpcoesDoRelatorioChangedEventArgs((OpcoesRelatorio)this.Clone()));
            }
        }

        #endregion

        #region CaminhoLogotipo

        public delegate void CaminhoLogotipoChangedEvent(object sender, CaminhoLogotipoChangedEventArgs e);

        public event CaminhoLogotipoChangedEvent CaminhoLogotipoChanged;

        public class CaminhoLogotipoChangedEventArgs : EventArgs
        {
            public CaminhoLogotipoChangedEventArgs(String caminho)
            {
                this.CaminhoLogotipo = caminho;
            }

            public String CaminhoLogotipo
            {
                get;
                set;
            }
        }

        private void informCaminhoLogotipoListeners()
        {
            if (CaminhoLogotipoChanged != null)
            {
                CaminhoLogotipoChanged(this, new CaminhoLogotipoChangedEventArgs(this.CaminhoLogotipo));
            }
        }

        #endregion

        #endregion

        #region Misc.

        public object Clone()
        {
            OpcoesRelatorio opcoes = new OpcoesRelatorio();
            opcoes.InformacoesAdicionais = this.InformacoesAdicionais;
            opcoes.ForcaMaxima = this.ForcaMaxima;
            opcoes.TensaoMaxima = this.TensaoMaxima;
            opcoes.ModuloElasticidade = this.ModuloElasticidade;
            opcoes.TensaoEscoamento = this.TensaoEscoamento;
            opcoes.ForcaEscoamento = this.ForcaEscoamento;
            opcoes.AreaInicial = this.AreaInicial;
            opcoes.InformacoesEmpresa = this.InformacoesEmpresa;
            opcoes.CaminhoLogotipo = this.CaminhoLogotipo;
            opcoes.Logotipo = this.Logotipo;
            return opcoes;
        }

        #endregion
    }
}
