﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using TestView.Ensaios.Utilidades;
using ModuloEnsaioTracao.Ensaios;
using TestView.Ensaios;

namespace ModuloEnsaioTracao.GUI
{
    public partial class FormInformacoesAdicionais : Form
    {
        #region Construtor e membros

        private BindingList<InformacaoAdicional> _informacoesAdicionais;
        private ModuloEnsaioTracao _moduloEnsaio;

        private EnsaioTracao EnsaioTracao
        {
            get
            {
                return (EnsaioTracao)this._moduloEnsaio.RotinaDeEnsaio.Ensaio;
            }
        }
        private RotinaDeEnsaioTracao RotinaEnsaioTracao
        {
            get
            {
                return (RotinaDeEnsaioTracao)this._moduloEnsaio.RotinaDeEnsaio;
            }
        }

        public FormInformacoesAdicionais(ModuloDeEnsaio moduloEnsaio)
        {
            InitializeComponent();
            this._moduloEnsaio = (ModuloEnsaioTracao)moduloEnsaio;
            this._informacoesAdicionais = new BindingList<InformacaoAdicional>(this.EnsaioTracao.InformacoesAdicionais);
            this.listBoxInformacoesAdicionais.DataSource = this._informacoesAdicionais;
            this.EnsaioTracao.EstadoEnsaioChanged += new Ensaio.EstadoEnsaioChangedEvent(EnsaioTracao_EstadoEnsaioChanged);
            this.habilitaInterface();
        }

        #endregion

        #region Lista de informações

        private void adicionarInformacao()
        {
            String novaInformacao = this.textBoxNovaInformacao.Text;
            if (!string.Empty.Equals(novaInformacao))
            {
                this.adicionarInformacao(novaInformacao);
                this.textBoxNovaInformacao.Text = string.Empty;
                
            }
        }

        private void adicionarInformacao(string novaInformacao)
        {
            this._informacoesAdicionais.Add(new InformacaoAdicional(novaInformacao, string.Empty));
        }

        private void salvarNovaListaInformacoes()
        {
            this.RotinaEnsaioTracao.substituiInformacoesAdicionais(this._informacoesAdicionais.ToList());
        }

        private void removerInformacao()
        {
            if (this.listBoxInformacoesAdicionais.SelectedItem != null)
            {
                this.removerInformacao((InformacaoAdicional)this.listBoxInformacoesAdicionais.SelectedItem);
            }
        }

        private void descerPosicao()
        {
            if (this.listBoxInformacoesAdicionais.SelectedItem != null)
            {
                InformacaoAdicional informacaoSelecionada = (InformacaoAdicional)this.listBoxInformacoesAdicionais.SelectedItem;
                this.moverInformacaoPosicao(this.listBoxInformacoesAdicionais.SelectedIndex, 1);
                this.listBoxInformacoesAdicionais.SelectedItem = informacaoSelecionada;
            }
        }

        private void subirPosicao()
        {
            if (this.listBoxInformacoesAdicionais.SelectedItem != null)
            {
                InformacaoAdicional informacaoSelecionada = (InformacaoAdicional)this.listBoxInformacoesAdicionais.SelectedItem;
                this.moverInformacaoPosicao(this.listBoxInformacoesAdicionais.SelectedIndex, -1);
                this.listBoxInformacoesAdicionais.SelectedItem = informacaoSelecionada;
            }
        }

        private void moverInformacaoPosicao(int indice, int direcao)
        {
            if (direcao > 0 && indice != this._informacoesAdicionais.Count - 1)
            {
                InformacaoAdicional buffer = this._informacoesAdicionais[indice + 1];
                this._informacoesAdicionais[indice + 1] = this._informacoesAdicionais[indice];
                this._informacoesAdicionais[indice] = buffer;
            }
            else if (direcao < 0 && indice != 0)
            {
                InformacaoAdicional buffer = this._informacoesAdicionais[indice - 1];
                this._informacoesAdicionais[indice - 1] = this._informacoesAdicionais[indice];
                this._informacoesAdicionais[indice] = buffer;
            }
        }

        private void removerInformacao(InformacaoAdicional informacao)
        {
            this._informacoesAdicionais.Remove(informacao);
        }

        #endregion

        #region Eventos

        private void buttonCancelar_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void buttonAdicionar_Click(object sender, EventArgs e)
        {
            this.adicionarInformacao();
        }

        private void buttonAplicar_Click(object sender, EventArgs e)
        {
            this.salvarNovaListaInformacoes();
        }

        private void buttonOk_Click(object sender, EventArgs e)
        {
            this.salvarNovaListaInformacoes();
            this.Close();
        }

        private void buttonRemover_Click(object sender, EventArgs e)
        {
            this.removerInformacao();
        }

        private void buttonBaixo_Click(object sender, EventArgs e)
        {
            this.descerPosicao();
        }

        private void buttonCima_Click(object sender, EventArgs e)
        {
            this.subirPosicao();
        }

        #endregion

        #region Misc.

        private void habilitaInterface(EstadoEnsaio estado)
        {
            if (estado == EstadoEnsaio.FECHADO)
            {
                this.groupBoxInformacoesAdicionais.Enabled = false;
                this.groupBoxNovaInformacao.Enabled = false;
            }
            else
            {
                this.groupBoxInformacoesAdicionais.Enabled = true;
                this.groupBoxNovaInformacao.Enabled = true;
            }
        }

        private void habilitaInterface()
        {
            this.habilitaInterface(this.EnsaioTracao.EstadoEnsaio);
        }

        #endregion

        #region Eventos

        void EnsaioTracao_EstadoEnsaioChanged(object sender, Ensaio.EstadoEnsaioEventArgs e)
        {
            this.habilitaInterface(e.EstadoEnsaio);
        }

        #endregion
    }
}
