﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ModuloEnsaioTracao.Ensaios;
using TestView.Utils;
using TestView.Ensaios.Utilidades.Instrumentacao;
using TestView.Ensaios;
using TestView.MyMath;

namespace ModuloEnsaioTracao.GUI
{
    public partial class FormOpcoesDeEnsaio : Form
    {
        #region Construtor e membros

        private ModuloEnsaioTracao _moduloEnsaio;
        private OpcoesEnsaioTracao _opcoesEnsaio;
        private Extensometro _extensometro;

        private EnsaioTracao EnsaioTracao
        {
            get
            {
                return (EnsaioTracao)this._moduloEnsaio.RotinaDeEnsaio.Ensaio;
            }
        }
        private RotinaDeEnsaioTracao RotinaEnsaioTracao
        {
            get
            {
                return (RotinaDeEnsaioTracao)this._moduloEnsaio.RotinaDeEnsaio;
            }
        }

        public FormOpcoesDeEnsaio(ModuloDeEnsaio moduloEnsaio)
        {
            InitializeComponent();
            this._moduloEnsaio = (ModuloEnsaioTracao)moduloEnsaio;
            this._opcoesEnsaio = (OpcoesEnsaioTracao)this.EnsaioTracao.OpcoesDeEnsaio.Clone();
            this._opcoesEnsaio.OpcaoModuloAjustado.ReferenciaChanged += new OpcaoModuloElasticidadeAjustado.ReferenciaChangedEvent(OpcaoModuloAjustado_ReferenciaChanged);
            if (this.EnsaioTracao.Extensometro != null)
            {
                this._extensometro = (Extensometro)this.EnsaioTracao.Extensometro.Clone();
            }
            this.habilitaInterface();
            this.inicializaCheckBoxes();
            this.inicializaTextBoxes();
            this.inicializaLabels();
            this.inicializaComboboxes();
        }

        #endregion

        #region Controles do form

        private void habilitaInterface()
        {
            EstadoEnsaio estado = this.EnsaioTracao.EstadoEnsaio;
            this.groupBoxComprimentoExtensometrico.Enabled = this._extensometro != null;
            this.groupBoxRemocaoExtensometro.Enabled = estado == EstadoEnsaio.NAO_INICIADO;
        }

        private void inicializaCheckBoxes()
        {
            this.checkBoxDetectarRompimento.Checked = this._opcoesEnsaio.DetectarRompimento;
        }

        #region Textboxes

        private void inicializaTextBoxes()
        {
            this.inicializaTextboxLimiarRompimento();
            this.inicializaTextboxesPontosModuloTempoReal();
            this.inicializaTextboxRetaParalelaExtensometro();
            this.inicializaTextboxComprimentoExtensometrico();
            this.inicializaTextboxesPontosModuloAjustado();
        }

        private void inicializaTextboxesPontosModuloAjustado()
        {
            ReferenciaAjusteModuloElasticidade referenciaModuloAjustado = this._opcoesEnsaio.OpcaoModuloAjustado.Referencia;
            if (referenciaModuloAjustado.GetType().Equals(typeof(PercentualFundoEscala)) || referenciaModuloAjustado.GetType().Equals(typeof(PercentualTensaoEscoamento)))
            {
                this.textBoxPontoInferiorModuloAjustado.Text = String.Format("{0}", referenciaModuloAjustado.PontoInferior * 100);
                this.textBoxPontoSuperiorModuloAjustado.Text = String.Format("{0}", referenciaModuloAjustado.PontoSuperior * 100);
            }
            else if (this._opcoesEnsaio.OpcaoModuloAjustado.Referencia.GetType().Equals(typeof(TensaoAbsoluta)))
            {
                this.textBoxPontoInferiorModuloAjustado.Text = String.Format("{0}", referenciaModuloAjustado.PontoInferior);
                this.textBoxPontoSuperiorModuloAjustado.Text = String.Format("{0}", referenciaModuloAjustado.PontoSuperior);
            }
        }

        private void inicializaTextboxComprimentoExtensometrico()
        {
            if (this._extensometro != null)
            {
                this.textBoxLe.Text = String.Format("{0}", this._extensometro.ComprimentoMedidaExtensometrico);
            }
        }

        private void inicializaTextboxRetaParalelaExtensometro()
        {
            this.textBoxRetaParalelaExtensometro.Text = String.Format("{0}", this._opcoesEnsaio.DistanciaParalelaExtensometro);
        }

        private void inicializaTextboxLimiarRompimento()
        {
            this.textBoxLimiarRompimento.Text = String.Format("{0}", this._opcoesEnsaio.LimiarDeRompimento * 100);
        }

        private void inicializaTextboxesPontosModuloTempoReal()
        {
            this.textBoxModuloTempoRealPontoSuperior.Text = String.Format("{0}", this._opcoesEnsaio.OpcaoModuloTempoReal.PontoFinal * 100);
            this.textBoxModuloTempoRealPontoInferior.Text = String.Format("{0}", this._opcoesEnsaio.OpcaoModuloTempoReal.PontoInicial * 100);
        }

        #endregion

        #region Comboboxes

        private void inicializaComboboxes()
        {
            this.inicializaComboboxReferenciaRemocaoExtensometro();
            this.inicializaComboboxReferenciaModuloAjustado();
            this.inicializaComboBoxUnidadeTensaoModuloAjustado();
        }

        private void inicializaComboboxReferenciaRemocaoExtensometro()
        {
            this.comboBoxReferenciaRemocao.Items.Clear();
            this.comboBoxReferenciaRemocao.Items.AddRange(new OpcoesRemocaoExtensometro[]{
                new RemocaoExtensometroParalela(),
                new RemocaoExtensometroDeformacao(),
                new RemocaoExtensometroPosicao(),
            });
            this.comboBoxReferenciaRemocao.SelectedIndexChanged += new EventHandler(comboBoxReferenciaRemocao_SelectedIndexChanged);
            foreach (OpcoesRemocaoExtensometro referenciaCombobox in this.comboBoxReferenciaRemocao.Items)
            {
                if (referenciaCombobox.GetType().Equals(this._opcoesEnsaio.OpcaoRemocaoExtensometro.GetType()))
                {
                    this.comboBoxReferenciaRemocao.SelectedItem = referenciaCombobox;
                }
            }
        }

        private void inicializaComboboxReferenciaModuloAjustado()
        {
            this.comboBoxReferenciaModuloAjustado.Items.Clear();
            this.comboBoxReferenciaModuloAjustado.Items.AddRange(new ReferenciaAjusteModuloElasticidade[]{
                new PercentualTensaoEscoamento(),
                new PercentualFundoEscala(),
                new TensaoAbsoluta(),
            });
            foreach (ReferenciaAjusteModuloElasticidade referenciaCombobox in this.comboBoxReferenciaModuloAjustado.Items)
            {
                if (referenciaCombobox.GetType().Equals(this._opcoesEnsaio.OpcaoModuloAjustado.Referencia.GetType()))
                {
                    this.comboBoxReferenciaModuloAjustado.SelectedItem = referenciaCombobox;
                }
            }
        }

        private void inicializaComboBoxUnidadeTensaoModuloAjustado()
        {
            this.comboBoxUnidadeTensaoModuloAjustado.Items.Clear();
            this.comboBoxUnidadeTensaoModuloAjustado.Items.AddRange(new Unidade[]{
                Unidade.MPA,
                Unidade.PSI,
                Unidade.KGF_MM2,
                Unidade.N_MM2,
            });
            if (this._opcoesEnsaio.OpcaoModuloAjustado.Referencia.GetType().Equals(typeof(TensaoAbsoluta)))
            {
                this.comboBoxUnidadeTensaoModuloAjustado.Enabled = true;
                TensaoAbsoluta referencia = (TensaoAbsoluta)this._opcoesEnsaio.OpcaoModuloAjustado.Referencia;
                foreach (Unidade unidade in this.comboBoxUnidadeTensaoModuloAjustado.Items)
                {
                    if (unidade.Equals(referencia.UnidadeTensao))
                    {
                        this.comboBoxUnidadeTensaoModuloAjustado.SelectedItem = unidade;
                    }
                }
            }
            else
            {
                this.comboBoxUnidadeTensaoModuloAjustado.Enabled = false;
            }
        }

        #endregion

        #region Labels

        private void inicializaLabels()
        {
            this.inicializaLabelUnidadeComprimentoExtensometrico();
            this.inicializaLabelsUnidadePontosModuloAjustado();
        }

        private void inicializaLabelsUnidadePontosModuloAjustado()
        {
            ReferenciaAjusteModuloElasticidade referencia = this._opcoesEnsaio.OpcaoModuloAjustado.Referencia;
            if (referencia.GetType().Equals(typeof(PercentualFundoEscala)))
            {
                this.labelUnidadePontoInferiorModuloAjustado.Text = Properties.Resources.percentualFundoEscalaUnidadeString;
                this.labelUnidadePontoSuperiorModuloAjustado.Text = Properties.Resources.percentualFundoEscalaUnidadeString;
            }
            else if (referencia.GetType().Equals(typeof(PercentualTensaoEscoamento)))
            {
                this.labelUnidadePontoInferiorModuloAjustado.Text = Properties.Resources.percentualTensaoEscoamentoUnidadeString;
                this.labelUnidadePontoSuperiorModuloAjustado.Text = Properties.Resources.percentualTensaoEscoamentoUnidadeString;
            }
            else if (referencia.GetType().Equals(typeof(TensaoAbsoluta)))
            {
                TensaoAbsoluta referenciaTensao = (TensaoAbsoluta)referencia;
                this.labelUnidadePontoInferiorModuloAjustado.Text = referenciaTensao.UnidadeTensao.ToString();
                this.labelUnidadePontoSuperiorModuloAjustado.Text = referenciaTensao.UnidadeTensao.ToString();
            }
        }

        private void inicializaLabelUnidadeComprimentoExtensometrico()
        {
            if (this._extensometro != null)
            {
                this.labelLe.Text = this._extensometro.UnidadeComprimento.ToString();
            }
        }

        #endregion

        #endregion

        #region Opcoes de ensaio

        private void salvaOpcoes() {
            this.RotinaEnsaioTracao.salvaOpcoesEnsaio(this._opcoesEnsaio);
            if (this._extensometro != null)
            {
                this.RotinaEnsaioTracao.atualizaExtensometro(this._extensometro);
            }
        }

        #endregion

        #region Eventos

        private void buttonCancelar_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void buttonAplicar_Click(object sender, EventArgs e)
        {
            this.salvaOpcoes();
        }

        private void buttonOk_Click(object sender, EventArgs e)
        {
            this.salvaOpcoes();
            this.Close();
        }

        private void checkBoxDetectarRompimento_CheckedChanged(object sender, EventArgs e)
        {
            CheckBox checkbox = (CheckBox)sender;
            this._opcoesEnsaio.DetectarRompimento = checkbox.Checked;
        }

        private void comboBoxReferenciaRemocao_SelectedIndexChanged(object sender, EventArgs e)
        {
            OpcoesRemocaoExtensometro referenciaRemocao = this._opcoesEnsaio.OpcaoRemocaoExtensometro;
            OpcoesRemocaoExtensometro referenciaCombobox = (OpcoesRemocaoExtensometro)this.comboBoxReferenciaRemocao.SelectedItem;
            if (!referenciaRemocao.GetType().Equals(referenciaCombobox.GetType()))
            {
                this._opcoesEnsaio.OpcaoRemocaoExtensometro = (OpcoesRemocaoExtensometro)referenciaCombobox.Clone();
            }
            this.textBoxLimiarRemocao.Text = String.Format("{0}", this._opcoesEnsaio.OpcaoRemocaoExtensometro.Limiar);
            this.labelRemoverExtensometroUnidade.Text = this._opcoesEnsaio.OpcaoRemocaoExtensometro.UnidadeLimiar.ToString();
        }

        private void comboBoxReferenciaModuloAjustado_SelectedIndexChanged(object sender, EventArgs e)
        {
            ReferenciaAjusteModuloElasticidade referenciaModuloAjustado = this._opcoesEnsaio.OpcaoModuloAjustado.Referencia;
            ReferenciaAjusteModuloElasticidade referenciaCombobox = (ReferenciaAjusteModuloElasticidade)this.comboBoxReferenciaModuloAjustado.SelectedItem;
            if (!referenciaModuloAjustado.GetType().Equals(referenciaCombobox.GetType()))
            {
                this._opcoesEnsaio.OpcaoModuloAjustado.Referencia = (ReferenciaAjusteModuloElasticidade)referenciaCombobox.Clone();
            }
        }

        private void OpcaoModuloAjustado_ReferenciaChanged(object sender, OpcaoModuloElasticidadeAjustado.ReferenciaChangedEventArgs e)
        {
            this.inicializaTextboxesPontosModuloAjustado();
            this.inicializaComboBoxUnidadeTensaoModuloAjustado();
            this.inicializaLabelsUnidadePontosModuloAjustado();
        }

        private void comboBoxUnidadeTensaoModuloAjustado_SelectedIndexChanged(object sender, EventArgs e)
        {
            ReferenciaAjusteModuloElasticidade referenciaModuloAjustado = this._opcoesEnsaio.OpcaoModuloAjustado.Referencia;
            if (referenciaModuloAjustado.GetType().Equals(typeof(TensaoAbsoluta)) && this.comboBoxUnidadeTensaoModuloAjustado.SelectedItem != null)
            {
                TensaoAbsoluta referenciaTensao = (TensaoAbsoluta)referenciaModuloAjustado;
                referenciaTensao.UnidadeTensao = (Unidade)this.comboBoxUnidadeTensaoModuloAjustado.SelectedItem;
                this.inicializaLabelsUnidadePontosModuloAjustado();
            }
        }

        #region Validação

        private void alternarAlertaValidacaoTextbox(TextBox textbox, string textoDica, bool alerta)
        {
            if (alerta)
            {
                textbox.BackColor = Color.LightSalmon;
                dica.Show(textoDica, textbox, textbox.Width, -20, 3000);
                textbox.SelectAll();
            }
            else
            {
                textbox.BackColor = SystemColors.Window;
            }
        }

        private void textboxLimiarRemocaoExtensometroValidating(object sender, CancelEventArgs e)
        {
            TextBox textbox = (TextBox)sender;
            string texto = textbox.Text;
            if (Validacao.validaValorNaoNulo(texto))
            {
                this.alternarAlertaValidacaoTextbox(textbox, null, false);
            }
            else
            {
                e.Cancel = true;
                this.alternarAlertaValidacaoTextbox(textbox, Properties.Resources.validaNumericoNaoNuloString, true);
            }
        }

        private void textboxPercentualValidating(object sender, CancelEventArgs e)
        {
            TextBox textbox = (TextBox)sender;
            string texto = textbox.Text;
            if (Validacao.validaTextoNumerico(texto))
            {
                double valor = double.Parse(texto);
                if (!Validacao.validaIntervaloValorNumerico(valor, 0, 100))
                {
                    e.Cancel = true;
                    this.alternarAlertaValidacaoTextbox(textbox, Properties.Resources.validaValorPercentualString, true);
                }
                else
                {
                    this.alternarAlertaValidacaoTextbox(textbox, null, false);
                }
            }
            else
            {
                e.Cancel = true;
                this.alternarAlertaValidacaoTextbox(textbox, Properties.Resources.validaValorPercentualString, true);
            }
        }

        private void textboxValorNaoNuloValidating(object sender, CancelEventArgs e)
        {
            TextBox textbox = (TextBox)sender;
            string texto = textbox.Text;
            if (Validacao.validaValorNaoNulo(texto))
            {
                this.alternarAlertaValidacaoTextbox(textbox, null, false);
            }
            else
            {
                e.Cancel = true;
                this.alternarAlertaValidacaoTextbox(textbox, Properties.Resources.validaNumericoNaoNuloString, true);
            }
        }

        private void textboxLimiarRompimentoValidated(object sender, EventArgs e)
        {
            TextBox textboxLimiar = (TextBox)sender;
            this._opcoesEnsaio.LimiarDeRompimento = double.Parse(textboxLimiar.Text) / 100.0;
        }

        private void textboxLimiarRemocaoExtensometroValidated(object sender, EventArgs e)
        {
            TextBox textboxLimiar = (TextBox)sender;
            OpcoesRemocaoExtensometro remocaoExtensometro = this._opcoesEnsaio.OpcaoRemocaoExtensometro;
            remocaoExtensometro.Limiar = double.Parse(textboxLimiar.Text);
        }

        private void textboxLeValidated(object sender, EventArgs e)
        {
            TextBox textboxLe = (TextBox)sender;
            this._extensometro.ComprimentoMedidaExtensometrico = double.Parse(textboxLe.Text);
        }

        private void textBoxRetaParalelaExtensometroValidated(object sender, EventArgs e)
        {
            TextBox textboxRetaParalelaExt = (TextBox)sender;
            this._opcoesEnsaio.DistanciaParalelaExtensometro = double.Parse(textboxRetaParalelaExt.Text);
        }

        private void textBoxPontoModuloAjustadoValidating(object sender, CancelEventArgs e)
        {
            TextBox textbox = (TextBox)sender;
            string texto = textbox.Text;
            ReferenciaAjusteModuloElasticidade referenciaModuloAjustado = this._opcoesEnsaio.OpcaoModuloAjustado.Referencia;
            if (Validacao.validaTextoNumerico(texto))
            {
                if (referenciaModuloAjustado.GetType().Equals(typeof(PercentualFundoEscala)) || referenciaModuloAjustado.GetType().Equals(typeof(PercentualTensaoEscoamento)))
                {
                    double valor = double.Parse(texto);
                    if (!Validacao.validaIntervaloValorNumerico(valor, 0, 100))
                    {
                        e.Cancel = true;
                        this.alternarAlertaValidacaoTextbox(textbox, Properties.Resources.validaValorPercentualString, true);
                    }
                    else
                    {
                        this.alternarAlertaValidacaoTextbox(textbox, null, false);
                    }
                }
                else
                {
                    this.alternarAlertaValidacaoTextbox(textbox, null, false);
                }
            }
            else
            {
                e.Cancel = true;
                this.alternarAlertaValidacaoTextbox(textbox, Properties.Resources.validaValorPercentualString, true);
            }
        }

        private void textBoxPontoInferiorModuloAjustadoValidated(object sender, EventArgs e)
        {
            TextBox textbox = (TextBox)sender;
            Double valor = Double.Parse(textbox.Text);
            ReferenciaAjusteModuloElasticidade referenciaModuloAjustado = this._opcoesEnsaio.OpcaoModuloAjustado.Referencia;
            if (referenciaModuloAjustado.GetType().Equals(typeof(PercentualFundoEscala)) || referenciaModuloAjustado.GetType().Equals(typeof(PercentualTensaoEscoamento)))
            {
                this._opcoesEnsaio.OpcaoModuloAjustado.Referencia.PontoInferior = valor / 100;
            }
            else
            {
                this._opcoesEnsaio.OpcaoModuloAjustado.Referencia.PontoInferior = valor;
            }
        }

        private void textBoxPontoSuperiorModuloAjustadoValidated(object sender, EventArgs e)
        {
            TextBox textbox = (TextBox)sender;
            Double valor = Double.Parse(textbox.Text);
            ReferenciaAjusteModuloElasticidade referenciaModuloAjustado = this._opcoesEnsaio.OpcaoModuloAjustado.Referencia;
            if (referenciaModuloAjustado.GetType().Equals(typeof(PercentualFundoEscala)) || referenciaModuloAjustado.GetType().Equals(typeof(PercentualTensaoEscoamento)))
            {
                this._opcoesEnsaio.OpcaoModuloAjustado.Referencia.PontoSuperior = valor / 100;
            }
            else
            {
                this._opcoesEnsaio.OpcaoModuloAjustado.Referencia.PontoSuperior = valor;
            }
        }

        #endregion

        #endregion
    }
}
