﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TestView.MyMath;
using TestView.Ensaios.Graficos;
using System.Runtime.Serialization;

namespace ModuloEnsaioTracao.Graficos
{
    [DataContract]
    public class OpcoesGraficoTracao : ICloneable
    {
        #region Construtor e membros

        private Unidade _unidadeAbscissa;
        private Unidade _unidadeOrdenada;
        private TipoCoordenada _tipoAbscissa;
        private TipoCoordenada _tipoOrdenada;
        private Boolean _exibirModuloElasticidade;
        private Boolean _exibirParalelaModulo;

        [DataMember]
        public Unidade UnidadeAbscissa
        {
            get
            {
                return this._unidadeAbscissa;
            }
            set
            {
                this._unidadeAbscissa = Unidade.MapaUnidades[value.Id];
            }
        }
        [DataMember]
        public Unidade UnidadeOrdenada
        {
            get
            {
                return this._unidadeOrdenada;
            }
            set
            {
                this._unidadeOrdenada = Unidade.MapaUnidades[value.Id];
            }
        }
        [DataMember]
        public TipoCoordenada TipoAbscissa
        {
            get
            {
                return this._tipoAbscissa;
            }
            set
            {
                this._tipoAbscissa = TipoCoordenada.MapaCoordenada[value.ToString()];
            }
        }
        [DataMember]
        public TipoCoordenada TipoOrdenada
        {
            get
            {
                return this._tipoOrdenada;
            }
            set
            {
                this._tipoOrdenada = TipoCoordenada.MapaCoordenada[value.ToString()];
            }
        }
        [DataMember]
        public Boolean ExibirModuloElasticidade
        {
            get
            {
                return this._exibirModuloElasticidade;
            }
            set
            {
                this._exibirModuloElasticidade = value;
            }
        }
        [DataMember]
        public Boolean ExibirParalelaModulo
        {
            get
            {
                return this._exibirParalelaModulo;
            }
            set
            {
                this._exibirParalelaModulo = value;
            }
        }

        public OpcoesGraficoTracao()
        {
            this.TipoAbscissa = TipoCoordenada.DEFORMACAO;
            this.TipoOrdenada = TipoCoordenada.TENSAO;
            this.UnidadeAbscissa = Unidade.MM;
            this.UnidadeOrdenada = Unidade.MPA;
            this.ExibirModuloElasticidade = true;
            this.ExibirParalelaModulo = true;
        }

        #endregion

        #region Salvar

        public void salvaOpcoesGrafico(OpcoesGraficoTracao opcoes)
        {
            this.TipoAbscissa = opcoes.TipoAbscissa;
            this.TipoOrdenada = opcoes.TipoOrdenada;
            this.UnidadeAbscissa = opcoes.UnidadeAbscissa;
            this.UnidadeOrdenada = opcoes.UnidadeOrdenada;
            this.ExibirModuloElasticidade = opcoes.ExibirModuloElasticidade;
            this.ExibirParalelaModulo = opcoes.ExibirParalelaModulo;
            this.informOpcoesDeGraficoListeners();
        }

        #endregion

        #region Eventos

        #region OpcoesDeGraficoChanged

        public delegate void OpcoesDeGraficoChangedEvent(object sender, OpcoesDeGraficoChangedEventArgs e);

        public event OpcoesDeGraficoChangedEvent OpcoesDeGraficoChanged;

        public class OpcoesDeGraficoChangedEventArgs : EventArgs
        {
            public OpcoesDeGraficoChangedEventArgs(TipoCoordenada tipoAbscissa, Unidade unidadeAbsissa, TipoCoordenada tipoOrdenada, Unidade unidadeOrdenada, Boolean exibirModuloElasticidade)
            {
                this.TipoAbscissa = tipoAbscissa;
                this.TipoOrdenada = tipoOrdenada;
                this.UnidadeAbscissa = unidadeAbsissa;
                this.UnidadeOrdenada = unidadeOrdenada;
                this.ExibirModuloElasticidade = exibirModuloElasticidade;
            }

            public TipoCoordenada TipoAbscissa
            {
                get;
                set;
            }

            public TipoCoordenada TipoOrdenada
            {
                get;
                set;
            }

            public Unidade UnidadeAbscissa
            {
                get;
                set;
            }

            public Unidade UnidadeOrdenada
            {
                get;
                set;
            }

            public Boolean ExibirModuloElasticidade
            {
                get;
                set;
            }
        }

        private void informOpcoesDeGraficoListeners()
        {
            if (OpcoesDeGraficoChanged != null)
            {
                OpcoesDeGraficoChanged(this, new OpcoesDeGraficoChangedEventArgs(this.TipoAbscissa, this.UnidadeAbscissa, this.TipoOrdenada, this.UnidadeOrdenada, this.ExibirModuloElasticidade));
            }
        }

        #endregion

        #endregion

        #region Misc.

        public object Clone()
        {
            OpcoesGraficoTracao opcoes = new OpcoesGraficoTracao();
            opcoes.TipoAbscissa = this.TipoAbscissa;
            opcoes.TipoOrdenada = this.TipoOrdenada;
            opcoes.UnidadeAbscissa = this.UnidadeAbscissa;
            opcoes.UnidadeOrdenada = this.UnidadeOrdenada;
            opcoes.ExibirParalelaModulo = this.ExibirParalelaModulo;
            opcoes.ExibirModuloElasticidade = this.ExibirModuloElasticidade;
            return opcoes;
        }

        #endregion
    }
}
