﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TestView.Ensaios;
using TestView.Conexoes;
using TestView.Condicionador;
using TestView;
using ModuloEnsaioTracao.Excecoes;
using System.Timers;
using TestView.Utils;
using TestView.Ensaios.Utilidades;
using TestView.MyMath;
using TestView.Ensaios.Utilidades.Coleta;
using ModuloEnsaioTracao.Graficos;
using TestView.Seguranca;
using TestView.Ensaios.Graficos;
using ModuloEnsaioTracao.Ensaios;
using TestView.Ensaios.Utilidades.CorposDeProva;
using TestView.Ensaios.Utilidades.Instrumentacao;
using System.IO;
using System.Drawing.Imaging;
using ModuloEnsaioTracao.Relatorios;
using System.Drawing.Printing;
using System.Drawing;

namespace ModuloEnsaioTracao
{
    public class RotinaDeEnsaioTracao : RotinaDeEnsaio
    {
        #region Construtor e membros

        private ModuloEnsaioTracao _moduloEnsaio;
        private EnsaioTracao _ensaio;
        private List<Conexao> _conexoes;
        private Temporizador _temporizadorColeta;
        private Dictionary<Conexao, SerieColeta> _seriesTemporais;
        private Conexao _conexaoForca, _conexaoDeformacao, _conexaoDeslocamento;

        public override ModuloDeEnsaio ModuloEnsaio
        {
            get
            {
                return this._moduloEnsaio;
            }
        }
        public override Ensaio Ensaio
        {
            get
            {
                return this._ensaio;
            }
            set
            {
                this._ensaio = (EnsaioTracao)value;
                this.assinaListeners();
            }
        }
        public EnsaioTracao EnsaioTracao
        {
            get
            {
                return (EnsaioTracao)this.Ensaio;
            }
        }
        public EstadoEnsaio EstadoDoEnsaio
        {
            get
            {
                return this.Ensaio.EstadoEnsaio;
            }
        }
        public ModuloTracaoGUI ModuloTracaoGUI
        {
            get
            {
                return (ModuloTracaoGUI)this._moduloEnsaio.FachadaGUI;
            }
        }

        public RotinaDeEnsaioTracao(ModuloEnsaioTracao moduloEnsaio)
        {
            this._moduloEnsaio = moduloEnsaio;
            this._ensaio = (EnsaioTracao)ModeloDeEnsaioTracao.getInstance().criaNovoEnsaio();
            this._ensaio.EstadoEnsaio = EstadoEnsaio.NAO_INICIADO;
            this._conexoes = new List<Conexao>();
            this._temporizadorColeta = new Temporizador(60, new Temporizador.FuncaoTemporizador(temporizador_Elapsed));
            this._seriesTemporais = new Dictionary<Conexao, SerieColeta>();
        }

        #endregion

        #region Ensaio

        private bool verificaConexoes()
        {
            bool transdutorForcaConectado = false;
            bool transdutorDeformacaoConectado = false;
            foreach (Conexao conexao in GerenciadorEnsaios.getInstance().Conexoes)
            {
                if (conexao.Conectado)
                {
                    Transdutor transdutor = conexao.CondicionadorSinal.Calibracao.TipoTransdutor;
                    if (transdutor.Equals(Transdutor.FORCA))
                    {
                        transdutorForcaConectado = true;
                    }
                    else if (transdutor.Equals(Transdutor.DEFORMACAO))
                    {
                        transdutorDeformacaoConectado = true;
                    }
                }
            }
            if (transdutorForcaConectado && transdutorDeformacaoConectado)
            {
                return true;
            }
            else
            {
                Logging.getInstance().info(Properties.Resources.transdutoresInsuficientesString, true);
                return false;
            }
        }

        private bool verificaExtensometro()
        {
            foreach (Conexao conexao in GerenciadorEnsaios.getInstance().Conexoes)
            {
                if (conexao.Conectado)
                {
                    Transdutor transdutor = conexao.CondicionadorSinal.Calibracao.TipoTransdutor;
                    if (transdutor == Transdutor.DEFORMACAO && !conexao.Instrumento.GetType().Equals(typeof(Extensometro)))
                    {
                        String mensagem = String.Format(Properties.Resources.extensometroNaoAssociadoString, conexao.PortaSerial.NomePorta);
                        TransdutoresInsuficientesException excecao = new TransdutoresInsuficientesException(mensagem);
                        Logging.getInstance().error(mensagem, true);
                        return false;
                    }
                }
            }
            return true;
        }

        private void alocaConexoes()
        {
            foreach (Conexao conexao in GerenciadorEnsaios.getInstance().Conexoes)
            {
                Transdutor transdutor = conexao.CondicionadorSinal.Calibracao.TipoTransdutor;
                if (transdutor.Equals(Transdutor.FORCA) && _conexaoForca == null)
                {
                    this._conexaoForca = conexao;
                    this._conexoes.Add(conexao);
                    conexao.ConexaoChanged += new Conexao.ConexaoChangedEvent(conexaoChanged);
                }
                else if (transdutor.Equals(Transdutor.DEFORMACAO) && _conexaoDeformacao == null)
                {
                    this._conexaoDeformacao = conexao;
                    this._conexoes.Add(conexao);
                    conexao.ConexaoChanged += new Conexao.ConexaoChangedEvent(conexaoChanged);
                }
                else if (transdutor.Equals(Transdutor.DESLOCAMENTO) && _conexaoDeslocamento == null)
                {
                    this._conexaoDeslocamento = conexao;
                    this._conexoes.Add(conexao);
                    conexao.ConexaoChanged += new Conexao.ConexaoChangedEvent(conexaoChanged);
                }
            }
        }

        private void criaSeriesTemporais()
        {
            foreach (Conexao conexao in this._conexoes)
            {
                Instrumento instrumento = (Instrumento)conexao.Instrumento.Clone();
                Calibracao calibracao = conexao.CondicionadorSinal.Calibracao;
                SerieColeta serie = new SerieColeta(calibracao, instrumento);
                this._ensaio.Aquisicao.adicionaSerieTemporal(serie);
                this._seriesTemporais.Add(conexao, serie);
            }
        }

        public override void iniciaEnsaioEspecifico()
        {
            if (this.EstadoDoEnsaio == EstadoEnsaio.NAO_INICIADO)
            {
                if (this.verificaConexoes() && this.verificaExtensometro())
                {
                    this.alocaConexoes();
                    this.criaSeriesTemporais();
                    this.assinaListeners();
                    this._ensaio.HoraInicioEnsaio = DateTime.Now;
                    this._temporizadorColeta.inicia();
                    this._ensaio.EstadoEnsaio = EstadoEnsaio.INICIADO;
                }
            }
        }

        public override void terminaEnsaioEspecifico()
        {
            if (this.EstadoDoEnsaio == EstadoEnsaio.INICIADO)
            {
                this._temporizadorColeta.para();
                this._ensaio.EstadoEnsaio = EstadoEnsaio.TERMINADO;
            }
        }

        public override void abrirEnsaioEspecifico(Ensaio ensaio)
        {
            this.Ensaio = (EnsaioTracao)ensaio;
            this.assinaListeners();
        }

        public void alternaEstadoEnsaio()
        {
            if (this.EstadoDoEnsaio == EstadoEnsaio.NAO_INICIADO)
            {
                this.iniciaEnsaio();
            }
            else if (this.EstadoDoEnsaio == EstadoEnsaio.INICIADO)
            {
                this.terminaEnsaio();
            }
        }

        private void coletaPontos()
        {
            DateTime horaColeta = DateTime.Now;
            List<Tuple<SerieColeta, Double>> coleta = new List<Tuple<SerieColeta, Double>>();
            foreach (Conexao conexao in this._conexoes)
            {
                Double valorColetado = conexao.Indicador.IndicacaoAjustada;
                SerieColeta serie = this._seriesTemporais[conexao];
                if (serie.Instrumento.GetType().Equals(typeof(Extensometro)) && this.EnsaioTracao.ExtensometroRemovido)
                {
                    double ultimaColeta = serie.PontosColetados[serie.PontosColetados.Count - 1].Valor;
                    coleta.Add(new Tuple<SerieColeta, Double>(serie, ultimaColeta));
                }
                else
                {
                    coleta.Add(new Tuple<SerieColeta, Double>(serie, valorColetado));
                }
            }
            this._ensaio.Aquisicao.adicionaConjuntoPontos(horaColeta, coleta);
        }

        private void analisaForcaMaxima(List<Tuple<SerieColeta, Double>> coleta)
        {
            foreach (Tuple<SerieColeta, Double> aquisicao in coleta)
            {
                if (aquisicao.Item1.Calibracao.TipoTransdutor.Equals(Transdutor.FORCA))
                {
                    if (double.IsNaN(this.EnsaioTracao.ForcaMaxima))
                    {
                        this.EnsaioTracao.ForcaMaxima = aquisicao.Item2;
                    }
                    else
                    {
                        this.EnsaioTracao.ForcaMaxima = Math.Max(this.EnsaioTracao.ForcaMaxima, aquisicao.Item2);
                    }
                    break;
                }
            }
        }

        private void detectarRompimento(List<Tuple<SerieColeta, Double>> coleta)
        {
            if (this.EnsaioTracao.ForcaMaxima > this.EnsaioTracao.SerieForca.Calibracao.Capacidade * 0.01 && !double.IsNaN(this.EnsaioTracao.ForcaMaxima))
            foreach (Tuple<SerieColeta, Double> aquisicao in coleta)
            {
                if (aquisicao.Item1.Calibracao.TipoTransdutor.Equals(Transdutor.FORCA))
                {
                    if (aquisicao.Item2 <= this.EnsaioTracao.ForcaMaxima * (1.0 - this.EnsaioTracao.OpcoesDeEnsaio.LimiarDeRompimento))
                    {
                        this.terminaEnsaio();
                    }
                }
            }
        }

        private void determinaVelocidadeForca()
        {
            // calcula a velocidade média dos últimos 1,5 segundos, e assume como velocidade instantânea
            double intervalo = 1500;
            if (this.EnsaioTracao.SerieForca.PontosColetados.Count > 1)
            {
                DateTime tempoUltimaColeta = this.EnsaioTracao.SerieForca.PontosColetados[this.EnsaioTracao.SerieForca.PontosColetados.Count-1].Tempo;
                DateTime tempoInicioIntervalo = tempoUltimaColeta;
                List<Ponto> pontosForca = new List<Ponto>();
                for (int i = this.EnsaioTracao.SerieForca.PontosColetados.Count - 1; i >= 0; i--)
                {
                    Double forca = this.EnsaioTracao.SerieForca.PontosColetados[i].Valor;
                    Double tempo = this.EnsaioTracao.SerieForca.PontosColetados[i].Tempo.TimeOfDay.TotalSeconds;
                    pontosForca.Add(new Ponto(tempo, forca));
                    tempoInicioIntervalo = this.EnsaioTracao.SerieForca.PontosColetados[i].Tempo;
                    if ((tempoUltimaColeta - tempoInicioIntervalo).TotalMilliseconds > intervalo)
                    {
                        break;
                    }
                }
                AjusteDeCurvaPolinomial ajuste = new AjusteDeCurvaPolinomial(pontosForca);
                Reta retaVelocidade = ajuste.ajustarReta();
                Double velocidade = Conversor.converterForca(retaVelocidade.Inclinacao, this.EnsaioTracao.SerieForca.Calibracao.Unidade, Unidade.KGF);
                this.EnsaioTracao.VelocidadeInstantaneaForca = velocidade;
            }
            else 
            {
                this.EnsaioTracao.VelocidadeInstantaneaForca = double.NaN;
            }
        }

        private void determinaVelocidadeDeslocamento()
        {
            // calcula a velocidade média dos últimos 1,5 segundos, e assume como velocidade instantânea
            double intervalo = 1500;
            if (this.EnsaioTracao.SerieDeslocamento != null)
            {
                if (this.EnsaioTracao.SerieDeslocamento.PontosColetados.Count > 1)
                {
                    DateTime tempoUltimaColeta = this.EnsaioTracao.SerieDeslocamento.PontosColetados[this.EnsaioTracao.SerieDeslocamento.PontosColetados.Count - 1].Tempo;
                    DateTime tempoInicioIntervalo = tempoUltimaColeta;
                    List<Ponto> pontosDeslocamento = new List<Ponto>();
                    for (int i = this.EnsaioTracao.SerieDeslocamento.PontosColetados.Count - 1; i >= 0; i--)
                    {
                        Double deslocamento = this.EnsaioTracao.SerieDeslocamento.PontosColetados[i].Valor;
                        Double tempo = this.EnsaioTracao.SerieDeslocamento.PontosColetados[i].Tempo.TimeOfDay.TotalSeconds;
                        pontosDeslocamento.Add(new Ponto(tempo, deslocamento));
                        tempoInicioIntervalo = this.EnsaioTracao.SerieDeslocamento.PontosColetados[i].Tempo;
                        if ((tempoUltimaColeta - tempoInicioIntervalo).TotalMilliseconds > intervalo)
                        {
                            break;
                        }
                    }
                    AjusteDeCurvaPolinomial ajuste = new AjusteDeCurvaPolinomial(pontosDeslocamento);
                    Reta retaVelocidade = ajuste.ajustarReta();
                    Double velocidade = Conversor.converterForca(retaVelocidade.Inclinacao, this.EnsaioTracao.SerieDeslocamento.Calibracao.Unidade, Unidade.MM);
                    this.EnsaioTracao.VelocidadeInstantaneaDeslocamento = velocidade;
                }
                else
                {
                    this.EnsaioTracao.VelocidadeInstantaneaDeslocamento = double.NaN;
                }
            }
        }

        private void remocaoExtensometro(List<Tuple<SerieColeta, Double>> coletas)
        {
            double comprimentoExt = this.EnsaioTracao.Extensometro.ComprimentoMedidaExtensometrico;
            double ultimaColetaDeformacao = double.NaN;
            double ultimaColetaForca = double.NaN;
            Unidade unidadeColetaDeformacao = null;
            Unidade unidadeColetaForca = null;
            foreach (Tuple<SerieColeta, Double> coleta in coletas)
            {
                if (coleta.Item1.Calibracao.TipoTransdutor.Equals(Transdutor.DEFORMACAO))
                {
                    ultimaColetaDeformacao = coleta.Item2;
                    unidadeColetaDeformacao = coleta.Item1.Calibracao.Unidade;
                }
                else if (coleta.Item1.Calibracao.TipoTransdutor.Equals(Transdutor.FORCA))
                {
                    ultimaColetaForca = coleta.Item2;
                    unidadeColetaForca = coleta.Item1.Calibracao.Unidade;
                }
            }

            OpcoesRemocaoExtensometro referenciaRemocao = this.EnsaioTracao.OpcoesDeEnsaio.OpcaoRemocaoExtensometro;
            if (referenciaRemocao.GetType().Equals(typeof(RemocaoExtensometroParalela)))
            {
                if (!double.IsNaN(this.EnsaioTracao.ModuloDeElasticidade) && !double.IsInfinity(this.EnsaioTracao.ModuloDeElasticidade))
                {
                    Double inclinacao = this.EnsaioTracao.ModuloDeElasticidade;
                    Ponto pontoParalelo = new Ponto(this.EnsaioTracao.OpcoesDeEnsaio.OpcaoRemocaoExtensometro.Limiar, 0.0);
                    Reta retaParalela = new Reta(pontoParalelo, inclinacao);

                    double ultimaTensao = Conversor.converterForca(ultimaColetaForca, unidadeColetaForca, Unidade.N) / this.EnsaioTracao.CorpoDeProva.AreaEquivalente;
                    double ultimaDeformacao = Conversor.converterComprimento(ultimaColetaDeformacao, unidadeColetaDeformacao, Unidade.PERCENTUAL_EXT, comprimentoExt);
                    if (ultimaTensao < retaParalela.obtemValorOrdenada(ultimaDeformacao))
                    {
                        this.EnsaioTracao.ExtensometroRemovido = true;
                    }
                }
            }
            else
            {
                if (referenciaRemocao.GetType().Equals(typeof(RemocaoExtensometroDeformacao)) || referenciaRemocao.GetType().Equals(typeof(RemocaoExtensometroPosicao)))
                {
                    double posicaoUltimaColeta = Conversor.converterComprimento(ultimaColetaDeformacao, unidadeColetaDeformacao, referenciaRemocao.UnidadeLimiar, comprimentoExt);
                    if (posicaoUltimaColeta > referenciaRemocao.Limiar)
                    {
                        this.EnsaioTracao.ExtensometroRemovido = true;
                    }
                }
            }
        }

        private void determinaTensaoEscoamento()
        {
            if (!double.IsNaN(this.EnsaioTracao.ModuloDeElasticidade) && !double.IsInfinity(this.EnsaioTracao.ModuloDeElasticidade))
            {
                Double distanciaParalela = this.EnsaioTracao.OpcoesDeEnsaio.DistanciaParalelaExtensometro;
                Reta retaParalela = new Reta(new Ponto(distanciaParalela, this.EnsaioTracao.InterseccaoOrdenadaModuloDeElasticidade), this.EnsaioTracao.ModuloDeElasticidade);
                Boolean retaParalelaAtingida = false;
                Double tensaoEscoamento = double.NaN;
                for (int i = 0; i < this.EnsaioTracao.SerieForca.PontosColetados.Count - 2; i++)
                {
                    Double tensaoInferior = Conversor.converterForca(this.EnsaioTracao.SerieForca.PontosColetados[i].Valor, this.EnsaioTracao.SerieForca.Calibracao.Unidade, Unidade.N) / this.EnsaioTracao.CorpoDeProva.AreaEquivalente;
                    Double deformacaoInferior = Conversor.converterComprimento(this.EnsaioTracao.SerieDeformacao.PontosColetados[i].Valor, this.EnsaioTracao.SerieDeformacao.Calibracao.Unidade, Unidade.PERCENTUAL_EXT, this.EnsaioTracao.Extensometro.ComprimentoMedidaExtensometrico);
                    Ponto pontoInferior = new Ponto(deformacaoInferior, tensaoInferior);

                    Double tensaoSuperior = Conversor.converterForca(this.EnsaioTracao.SerieForca.PontosColetados[i + 1].Valor, this.EnsaioTracao.SerieForca.Calibracao.Unidade, Unidade.N) / this.EnsaioTracao.CorpoDeProva.AreaEquivalente;
                    Double deformacaoSuperior = Conversor.converterComprimento(this.EnsaioTracao.SerieDeformacao.PontosColetados[i + 1].Valor, this.EnsaioTracao.SerieDeformacao.Calibracao.Unidade, Unidade.PERCENTUAL_EXT, this.EnsaioTracao.Extensometro.ComprimentoMedidaExtensometrico);
                    Ponto pontoSuperior = new Ponto(deformacaoSuperior, tensaoSuperior);

                    Reta segmento = new Reta(pontoInferior, pontoSuperior);
                    Ponto interseccao = retaParalela.obtemInterseccao(segmento);
                    if (interseccao != null)
                    {
                        if ((interseccao.X >= pontoInferior.X && interseccao.X <= pontoSuperior.X) || (interseccao.X <= pontoInferior.X && interseccao.X >= pontoSuperior.X))
                        {
                            tensaoEscoamento = interseccao.Y;
                            retaParalelaAtingida = true;
                        }
                    }
                }
                this.EnsaioTracao.RetaParalelaModuloAtingida = retaParalelaAtingida;
                this.EnsaioTracao.TensaoEscoamento = tensaoEscoamento;
            }
            else
            {
                this.EnsaioTracao.RetaParalelaModuloAtingida = false;
                this.EnsaioTracao.TensaoEscoamento = double.NaN;
            }
        }

        private void rotinaPosEnsaio()
        {
            this.determinarModuloElasticidade();
            this.determinaTensaoEscoamento();
            this.redesenhaSeries();
        }

        #endregion

        #region Relatório

        public void gerarRelatorioODT(string nomeArquivo)
        {
            RelatorioODTEnsaioTracao relatorio = new RelatorioODTEnsaioTracao(this.EnsaioTracao);
            relatorio.salvarRelatorio(nomeArquivo);
        }

        #endregion

        #region Impressao

        public override void imprimeRelatorio(PrintPageEventArgs e)
        {
            RelatorioEnsaioTracao relatorio = this.EnsaioTracao.RelatorioEnsaioTracao;

            Font fonteTitulo = new Font(FontFamily.GenericSansSerif, 36);
            Font fontePequena = new Font(FontFamily.GenericSansSerif, 11);
            Font fonteCoum = new Font(FontFamily.GenericSansSerif, 14);
            Brush pincelPreto = new SolidBrush(Color.Black);

            #region Cabeçalho

            e.Graphics.DrawString(Properties.Resources.relatorioEnsaioString, fonteTitulo, pincelPreto, 10, 10);
            e.Graphics.DrawString(relatorio.OpcoesDoRelatorio.InformacoesEmpresa, fontePequena, pincelPreto, 20, 70);

            #endregion

            #region Data ensaio

            String dataEnsaio = String.Format("{0}: {1}", Properties.Resources.dataEnsaioString, this.EnsaioTracao.DataEnsaio);
            e.Graphics.DrawString(dataEnsaio, fontePequena, pincelPreto, 20, 160);

            #endregion

            #region Gráfico ensaio

            e.Graphics.DrawImage(this.ModuloTracaoGUI.ImagemGrafico, 20, 180, 630, 620);

            #endregion

            #region Informações Adicionais

            if (this.EnsaioTracao.RelatorioEnsaioTracao.OpcoesDoRelatorio.InformacoesAdicionais)
            {
                String informacoesAdicionais = String.Empty;
                if (this.EnsaioTracao.RelatorioEnsaioTracao.OpcoesDoRelatorio.InformacoesAdicionais)
                {
                    for (int i = 0; i < this.EnsaioTracao.InformacoesAdicionais.Count; i++)
                    {
                        InformacaoAdicional informacao = this.EnsaioTracao.InformacoesAdicionais[i];
                        String stringInformacao = String.Format("{0}: {1}", informacao.Informacao, informacao.Valor);
                        informacoesAdicionais += stringInformacao + Environment.NewLine;
                    }
                }
                e.Graphics.DrawString(informacoesAdicionais, fonteCoum, pincelPreto, 665, 180);
            }
            
            #endregion

            #region Dados do ensaio

            String dadosEnsaio = String.Empty;
            OpcoesRelatorio opcoes = this.EnsaioTracao.RelatorioEnsaioTracao.OpcoesDoRelatorio;
            #region Força máxima
            if (opcoes.ForcaMaxima)
            {
                String forcaMaximaString = String.Format("{0}: {1}", Properties.Resources.forcaMaximaString, Properties.Resources.indicacaoVaziaString);
                Double forcaMaxima = this.EnsaioTracao.ForcaMaxima;
                if (!double.IsNaN(forcaMaxima))
                {
                    Unidade unidadeForcaMaxima = this.EnsaioTracao.UnidadeForcaMaxima;
                    forcaMaximaString = String.Format("{0}: {1:0.00} {2}", Properties.Resources.forcaMaximaString, forcaMaxima, unidadeForcaMaxima.ToString());
                }
                dadosEnsaio += Environment.NewLine + forcaMaximaString;
            }
            #endregion
            #region Tensão máxima
            if (opcoes.TensaoMaxima)
            {
                String tensaoMaximaString = String.Format("{0}: {1}", Properties.Resources.tensaoMaximaString, Properties.Resources.indicacaoVaziaString);
                Double tensaoMaxima = this.EnsaioTracao.TensaoMaxima;
                if (!double.IsNaN(tensaoMaxima))
                {
                    Unidade unidadeTensaoMaxima = this.EnsaioTracao.UnidadeTensaoMaxima;
                    tensaoMaximaString = String.Format("{0}: {1:0.00} {2}", Properties.Resources.tensaoMaximaString, tensaoMaxima, unidadeTensaoMaxima.ToString());
                }
                dadosEnsaio += Environment.NewLine + tensaoMaximaString;
            }
            #endregion
            #region Módulo de elasticidade
            if (opcoes.ModuloElasticidade)
            {
                String moduloElasticidadeString = String.Format("{0}: {1}", Properties.Resources.moduloElasticidadeString, Properties.Resources.indicacaoVaziaString);
                Double modulo = this.EnsaioTracao.ModuloDeElasticidade;
                if (!double.IsNaN(modulo))
                {
                    Unidade unidadeModulo = this.EnsaioTracao.UnidadeModuloElasticidade;
                    Double moduloGPa = Conversor.converterTensao(modulo, unidadeModulo, Unidade.GPA);
                    moduloElasticidadeString = String.Format("{0}: {1:0.00} {2}", Properties.Resources.moduloElasticidadeString, moduloGPa, Unidade.GPA.ToString());
                }
                dadosEnsaio += Environment.NewLine + moduloElasticidadeString;
            }
            #endregion
            #region Tensão escoamento
            if (opcoes.TensaoEscoamento)
            {
                String tensaoEscoamentoString = String.Format("{0}: {1}", Properties.Resources.tensaoEscoamentoString, Properties.Resources.indicacaoVaziaString);
                Double tensaoEscoamento = this.EnsaioTracao.TensaoEscoamento;
                if (!double.IsNaN(tensaoEscoamento))
                {
                    Unidade unidadeTensaoEscoamento = this.EnsaioTracao.UnidadeTensaoEscoamento;
                    tensaoEscoamentoString = String.Format("{0}: {1:0.00} {2}", Properties.Resources.tensaoEscoamentoString, tensaoEscoamento, unidadeTensaoEscoamento.ToString());
                }
                dadosEnsaio += Environment.NewLine + tensaoEscoamentoString;
            }
            #endregion
            #region Força escoamento
            if (opcoes.ForcaEscoamento)
            {
                String forcaEscoamentoString = String.Format("{0}: {1}", Properties.Resources.forcaEscoamentoString, Properties.Resources.indicacaoVaziaString);
                Double forcaEscoamento = this.EnsaioTracao.ForcaEscoamento;
                if (!double.IsNaN(forcaEscoamento))
                {
                    Unidade unidadeForcaEscoamento = this.EnsaioTracao.UnidadeForcaEscoamento;
                    forcaEscoamentoString = String.Format("{0}: {1:0.00} {2}", Properties.Resources.forcaEscoamentoString, forcaEscoamento, unidadeForcaEscoamento.ToString());
                }
                dadosEnsaio += Environment.NewLine + forcaEscoamentoString;
            }
            #endregion
            #region Área Inicial
            if (opcoes.AreaInicial)
            {
                String areaInicialString = String.Format("{0}: {1}", Properties.Resources.areaInicialString, Properties.Resources.indicacaoVaziaString);
                Double areaInicial = this.EnsaioTracao.CorpoDeProva.AreaEquivalente;
                if (!double.IsNaN(areaInicial))
                {
                    Unidade unidadeArea = Unidade.MM2;
                    areaInicialString = String.Format("{0}: {1:0.00} {2}", Properties.Resources.areaInicialString, areaInicial, unidadeArea.ToString());
                }
                dadosEnsaio += Environment.NewLine + areaInicialString;
            }
            #endregion
            e.Graphics.DrawString(dadosEnsaio, fonteCoum, pincelPreto, 665, 550);

            #endregion
        }

        #endregion

        #region Informacao adicional

        public void salvaInformacaoAdicional(InformacaoAdicional informacao, String valor)
        {
            informacao.Valor = valor;
        }

        public void substituiInformacoesAdicionais(List<InformacaoAdicional> informacoes)
        {
            this.EnsaioTracao.InformacoesAdicionais = informacoes.Select(x => (InformacaoAdicional)x.Clone()).ToList();
        }

        #endregion

        #region Módulo de elasticidade

        private void determinarModuloElasticidade()
        {
            if (this.EnsaioTracao.EstadoEnsaio == EstadoEnsaio.INICIADO)
            {
                this.determinarModuloElasticidadeTempoReal();
            }
            else if (this.EnsaioTracao.EstadoEnsaio == EstadoEnsaio.TERMINADO || this.EnsaioTracao.EstadoEnsaio == EstadoEnsaio.FECHADO)
            {
                this.determinarModuloElasticidadeAjustado();
            }
        }

        private void determinarModuloElasticidadeTempoReal()
        {
            double valorMaximoGrafico = this.ModuloTracaoGUI.MaximoOrdenada;
            double valorMaximoGraficoForca = double.NaN;
            double areaCorpoProva = this.EnsaioTracao.CorpoDeProva.AreaEquivalente;

            SerieColeta serieForca = this.EnsaioTracao.SerieForca, serieDeformacao = this.EnsaioTracao.SerieDeformacao;

            if (serieDeformacao != null)
            {
                if (this.EnsaioTracao.Grafico.OpcoesGrafico.TipoOrdenada.Equals(TipoCoordenada.TENSAO))
                {
                    double valorMaximoGraficoTensaoMPA = Conversor.converterTensao(valorMaximoGrafico, this.EnsaioTracao.Grafico.OpcoesGrafico.UnidadeOrdenada, Unidade.MPA);
                    valorMaximoGraficoForca = Conversor.converterForca(valorMaximoGraficoTensaoMPA * areaCorpoProva, Unidade.N, serieForca.Calibracao.Unidade);
                }
                else
                {
                    valorMaximoGraficoForca = Conversor.converterForca(valorMaximoGrafico, this.EnsaioTracao.Grafico.OpcoesGrafico.UnidadeOrdenada, serieForca.Calibracao.Unidade);
                }

                double limiarInferior = this.EnsaioTracao.OpcoesDeEnsaio.OpcaoModuloTempoReal.PontoInicial * valorMaximoGraficoForca;
                double limiarSuperior = this.EnsaioTracao.OpcoesDeEnsaio.OpcaoModuloTempoReal.PontoFinal * valorMaximoGraficoForca;

                Ponto pontoInferior = null, pontoSuperior = null;
                #region Obtem pontos inferior e superior do modulo de elasticidade em tempo real
                for (int i = 0; i < serieForca.PontosColetados.Count - 1; i++)
                {
                    PontoColetado pontoForca1 = serieForca.PontosColetados[i];
                    PontoColetado pontoForca2 = serieForca.PontosColetados[i + 1];
                    PontoColetado pontoDeformacao1 = serieDeformacao.PontosColetados[i];
                    PontoColetado pontoDeformacao2 = serieDeformacao.PontosColetados[i + 1];
                    if (pontoInferior == null)
                    {
                        if ((double)pontoForca1.Valor <= limiarInferior && (double)pontoForca2.Valor >= limiarInferior)
                        {
                            if (Math.Abs((double)pontoForca1.Valor - limiarInferior) <= Math.Abs((double)pontoForca2.Valor - limiarInferior))
                            {
                                pontoInferior = new Ponto(pontoDeformacao1.Valor, pontoForca1.Valor);
                            }
                            else
                            {
                                pontoInferior = new Ponto(pontoDeformacao2.Valor, pontoForca2.Valor);
                            }
                        }
                    }
                    if (pontoSuperior == null)
                    {
                        if ((double)pontoForca1.Valor <= limiarSuperior && (double)pontoForca2.Valor >= limiarSuperior)
                        {
                            if (Math.Abs((double)pontoForca1.Valor - limiarSuperior) <= Math.Abs((double)pontoForca2.Valor - limiarSuperior))
                            {
                                pontoSuperior = new Ponto(pontoDeformacao1.Valor, pontoForca1.Valor);
                            }
                            else
                            {
                                pontoSuperior = new Ponto(pontoDeformacao2.Valor, pontoForca2.Valor);
                            }
                        }
                    }
                    if (pontoInferior != null && pontoSuperior != null)
                    {
                        break;
                    }
                }
                #endregion

                #region Calcula modulo de elasticidade

                if (pontoInferior != null && pontoSuperior != null)
                {
                    pontoInferior.X = pontoInferior.X / this.EnsaioTracao.Extensometro.ComprimentoMedidaExtensometrico;
                    pontoSuperior.X = pontoSuperior.X / this.EnsaioTracao.Extensometro.ComprimentoMedidaExtensometrico;
                    pontoInferior.Y = Conversor.converterForca(pontoInferior.Y, serieForca.Calibracao.Unidade, Unidade.N) / areaCorpoProva;
                    pontoSuperior.Y = Conversor.converterForca(pontoSuperior.Y, serieForca.Calibracao.Unidade, Unidade.N) / areaCorpoProva;
                    Reta retaModulo = new Reta(pontoInferior, pontoSuperior);
                    this.EnsaioTracao.ModuloDeElasticidade = retaModulo.Inclinacao;
                    this.EnsaioTracao.InterseccaoOrdenadaModuloDeElasticidade = retaModulo.obtemValorOrdenada(0.0);
                }
                else
                {
                    this.EnsaioTracao.ModuloDeElasticidade = double.NaN;
                    this.EnsaioTracao.InterseccaoOrdenadaModuloDeElasticidade = double.NaN;
                }

                #endregion
            }
            else
            {
                this.EnsaioTracao.ModuloDeElasticidade = double.NaN;
                this.EnsaioTracao.InterseccaoOrdenadaModuloDeElasticidade = double.NaN;
            }
        }

        private void determinarModuloElasticidadeAjustado()
        {
            ReferenciaAjusteModuloElasticidade referenciaAjuste = this.EnsaioTracao.OpcoesDeEnsaio.OpcaoModuloAjustado.Referencia;
            List<Ponto> pontos = new List<Ponto>();
            CorpoDeProva corpoProva = this.EnsaioTracao.CorpoDeProva;
            Extensometro extensometro = this.EnsaioTracao.Extensometro;
            SerieColeta serieForca = this.EnsaioTracao.SerieForca;
            SerieColeta serieDeformacao = this.EnsaioTracao.SerieDeformacao;

            double tensaoInferior = double.NaN;
            double tensaoSuperior = double.NaN;
            if (referenciaAjuste.GetType().Equals(typeof(TensaoAbsoluta)))
            {
                TensaoAbsoluta referenciaTensao = (TensaoAbsoluta)referenciaAjuste;
                tensaoInferior = Conversor.converterTensao(referenciaTensao.PontoInferior, referenciaTensao.UnidadeTensao, Unidade.MPA);
                tensaoSuperior = Conversor.converterTensao(referenciaTensao.PontoSuperior, referenciaTensao.UnidadeTensao, Unidade.MPA);
            }
            else if (referenciaAjuste.GetType().Equals(typeof(PercentualTensaoEscoamento)) && !double.IsNaN(this.EnsaioTracao.TensaoEscoamento))
            {
                tensaoInferior = this.EnsaioTracao.TensaoEscoamento * referenciaAjuste.PontoInferior;
                tensaoSuperior = this.EnsaioTracao.TensaoEscoamento * referenciaAjuste.PontoSuperior;
            }
            else if (referenciaAjuste.GetType().Equals(typeof(PercentualFundoEscala)))
            {
                double tensaoFundoEscala = double.NaN;
                if (this.EnsaioTracao.Grafico.OpcoesGrafico.TipoOrdenada.Equals(TipoCoordenada.FORCA))
                {
                    tensaoFundoEscala = Conversor.converterForca(this.ModuloTracaoGUI.MaximoOrdenada, this.EnsaioTracao.Grafico.OpcoesGrafico.UnidadeOrdenada, Unidade.N) / corpoProva.AreaEquivalente;
                }
                else
                {
                    tensaoFundoEscala = Conversor.converterTensao(this.ModuloTracaoGUI.MaximoOrdenada, this.EnsaioTracao.Grafico.OpcoesGrafico.UnidadeOrdenada, Unidade.MPA);
                }
                tensaoInferior = tensaoFundoEscala * referenciaAjuste.PontoInferior;
                tensaoSuperior = tensaoFundoEscala * referenciaAjuste.PontoSuperior;
            }
            if (!double.IsNaN(tensaoInferior) && !double.IsNaN(tensaoSuperior))
            {
                for (int i = 0; i < serieForca.PontosColetados.Count; i++)
                {
                    double tensao = Conversor.converterForca(serieForca.PontosColetados[i].Valor, serieForca.Calibracao.Unidade, Unidade.N) / corpoProva.AreaEquivalente;
                    if ((tensaoInferior <= tensao && tensao <= tensaoSuperior) || (tensaoInferior >= tensao && tensao >= tensaoSuperior))
                    {
                        double deformacao = Conversor.converterComprimento(serieDeformacao.PontosColetados[i].Valor, serieDeformacao.Calibracao.Unidade, Unidade.PERCENTUAL_EXT, extensometro.ComprimentoMedidaExtensometrico);
                        Ponto ponto = new Ponto(deformacao, tensao);
                        pontos.Add(ponto);
                    }
                    else if (pontos.Count > 0)
                    {
                        break;
                    }
                }
                if (pontos.Count >= 2)
                {
                    AjusteDeCurvaPolinomial ajusteCurva = new AjusteDeCurvaPolinomial(pontos);
                    Reta retaAjustada = ajusteCurva.ajustarReta();

                    this.EnsaioTracao.ModuloDeElasticidade = retaAjustada.Inclinacao;
                    this.EnsaioTracao.InterseccaoOrdenadaModuloDeElasticidade = retaAjustada.obtemValorOrdenada(0);
                }
            }
        }

        #endregion

        #region Corpo de prova

        public void selecionaCorpoDeProva(CorpoDeProva seccao)
        {
            if (!seccao.GetType().Equals(this.EnsaioTracao.CorpoDeProva.GetType()))
            {
                this.EnsaioTracao.CorpoDeProva = (CorpoDeProva)seccao.Clone();
            }
        }

        #endregion

        #region Grafico

        private const int MAXIMO_PONTOS_GRAFICO = 100;

        #region Series

        private void redesenhaSeries()
        {
            lock (this.EnsaioTracao.Grafico.SeriePrincipal)
            {
                this.redesenhaSeriePrincipal();
                this.redesenhaSerieModuloElasticidade();
                this.redesenhaSerieParalelaModulo();
            }
            this.EnsaioTracao.Grafico.informGraficoChangedListeners();
        }

        #region Serie Principal

        private void redesenhaSeriePrincipal()
        {
            this.EnsaioTracao.Grafico.SeriePrincipal.Clear();
            List<PontoGrafico> seriePrincipal = new List<PontoGrafico>();

            int numeroPontosColetados = this.EnsaioTracao.Aquisicao.QuantidadePontosColetados;
            int distancia;
            int indicePonto;
            for (distancia = 1; numeroPontosColetados / distancia > MAXIMO_PONTOS_GRAFICO; distancia++) ;

            for (indicePonto = 0; indicePonto < numeroPontosColetados; )
            {
                Double pontoOrdenada = double.NaN;
                Double pontoAbscissa = double.NaN;

                DateTime horaPonto = this.EnsaioTracao.HoraInicioEnsaio;

                foreach (SerieColeta serie in this.EnsaioTracao.Aquisicao.SeriesTemporais)
                {
                    horaPonto = serie.PontosColetados[indicePonto].Tempo;
                    if (double.IsNaN(pontoOrdenada))
                    {
                        if (this.EnsaioTracao.Grafico.OpcoesGrafico.TipoOrdenada == TipoCoordenada.FORCA && serie.Calibracao.TipoTransdutor == Transdutor.FORCA)
                        {
                            double valor = serie.PontosColetados[indicePonto].Valor;
                            valor = Conversor.converterForca(valor, serie.Calibracao.Unidade, this.EnsaioTracao.Grafico.OpcoesGrafico.UnidadeOrdenada);
                            pontoOrdenada = valor;
                        }

                        else if (this.EnsaioTracao.Grafico.OpcoesGrafico.TipoOrdenada == TipoCoordenada.TENSAO && serie.Calibracao.TipoTransdutor == Transdutor.FORCA)
                        {
                            double valor = serie.PontosColetados[indicePonto].Valor;
                            double valorForcaN = Conversor.converterForca(valor, serie.Calibracao.Unidade, Unidade.N);
                            double valorTensaoMPa = valorForcaN / this.EnsaioTracao.CorpoDeProva.AreaEquivalente;
                            double valorTensao = Conversor.converterTensao(valorTensaoMPa, Unidade.MPA, this.EnsaioTracao.Grafico.OpcoesGrafico.UnidadeOrdenada);
                            pontoOrdenada = valorTensao;
                        }
                    }
                    if (double.IsNaN(pontoAbscissa))
                    {
                        if (this.EnsaioTracao.Grafico.OpcoesGrafico.TipoAbscissa == TipoCoordenada.DEFORMACAO && serie.Calibracao.TipoTransdutor == Transdutor.DEFORMACAO)
                        {
                            double valor = serie.PontosColetados[indicePonto].Valor;
                            if (serie.Instrumento.GetType().Equals(typeof(Extensometro)))
                            {
                                Extensometro extensometro = (Extensometro)serie.Instrumento;
                                valor = Conversor.converterComprimento(valor, serie.Calibracao.Unidade, this.EnsaioTracao.Grafico.OpcoesGrafico.UnidadeAbscissa, extensometro.ComprimentoMedidaExtensometrico);
                            }
                            else
                            {
                                valor = Conversor.converterComprimento(valor, serie.Calibracao.Unidade, this.EnsaioTracao.Grafico.OpcoesGrafico.UnidadeAbscissa, double.NaN);
                            }
                            pontoAbscissa = valor;
                        }
                        else if (this.EnsaioTracao.Grafico.OpcoesGrafico.TipoAbscissa == TipoCoordenada.DESLOCAMENTO && serie.Calibracao.TipoTransdutor == Transdutor.DESLOCAMENTO)
                        {
                            double valor = serie.PontosColetados[indicePonto].Valor;
                            valor = Conversor.converterComprimento(valor, serie.Calibracao.Unidade, this.EnsaioTracao.Grafico.OpcoesGrafico.UnidadeAbscissa, double.NaN);
                            pontoAbscissa = valor;
                        }
                    }
                }

                if (this.EnsaioTracao.Grafico.OpcoesGrafico.TipoAbscissa == TipoCoordenada.TEMPO)
                {
                    TimeSpan intervalo = horaPonto - this.EnsaioTracao.HoraInicioEnsaio;
                    if (this.EnsaioTracao.Grafico.OpcoesGrafico.UnidadeAbscissa == Unidade.SEGUNDOS)
                    {
                        pontoAbscissa = intervalo.TotalSeconds;
                    }
                }

                if (this.EnsaioTracao.Grafico.OpcoesGrafico.TipoOrdenada == TipoCoordenada.TEMPO)
                {
                    TimeSpan intervalo = horaPonto - this.EnsaioTracao.HoraInicioEnsaio;
                    if (this.EnsaioTracao.Grafico.OpcoesGrafico.UnidadeOrdenada == Unidade.SEGUNDOS)
                    {
                        pontoOrdenada = intervalo.TotalSeconds;
                    }
                }

                seriePrincipal.Add(new PontoGrafico(pontoAbscissa, pontoOrdenada));

                if (indicePonto + distancia > numeroPontosColetados && indicePonto != numeroPontosColetados - 1)
                {
                    indicePonto = numeroPontosColetados - 1;
                }
                else
                {
                    indicePonto += distancia;
                }
            }

            this.EnsaioTracao.Grafico.SeriePrincipal = seriePrincipal;
        }

        #endregion

        #region Serie Modulo de Elasticidade

        private void redesenhaSerieModuloElasticidade()
        {
            this.EnsaioTracao.Grafico.SerieModuloElasticidade.Clear();
            List<PontoGrafico> serieModulo = new List<PontoGrafico>();

            if (this.EnsaioTracao.Grafico.OpcoesGrafico.ExibirModuloElasticidade) {
                if (!double.IsNaN(this.EnsaioTracao.ModuloDeElasticidade) && !double.IsNaN(this.EnsaioTracao.InterseccaoOrdenadaModuloDeElasticidade) &&
                    !double.IsInfinity(this.EnsaioTracao.ModuloDeElasticidade) && !double.IsInfinity(this.EnsaioTracao.InterseccaoOrdenadaModuloDeElasticidade))
                {
                    if (this.EnsaioTracao.Grafico.OpcoesGrafico.TipoOrdenada.Equals(TipoCoordenada.TENSAO) && this.EnsaioTracao.Grafico.OpcoesGrafico.TipoAbscissa.Equals(TipoCoordenada.DEFORMACAO))
                    {
                        Extensometro extensometro = this.EnsaioTracao.Extensometro;

                        double interseccao = Conversor.converterTensao(this.EnsaioTracao.InterseccaoOrdenadaModuloDeElasticidade, Unidade.MPA, this.EnsaioTracao.Grafico.OpcoesGrafico.UnidadeOrdenada);
                        double inclinacao = Conversor.converterTensao(this.EnsaioTracao.ModuloDeElasticidade, Unidade.MPA, this.EnsaioTracao.Grafico.OpcoesGrafico.UnidadeOrdenada);
                        if (this.EnsaioTracao.Grafico.OpcoesGrafico.UnidadeAbscissa.Equals(Unidade.MM))
                        {
                            inclinacao /= extensometro.ComprimentoMedidaExtensometrico;
                        }

                        Reta retaModulo = new Reta(new Ponto(0.0, interseccao), inclinacao);

                        PontoGrafico pontoModuloInferior = new PontoGrafico(0.0, interseccao);

                        double tensaoMaxima = Conversor.converterTensao(this.EnsaioTracao.TensaoMaxima, Unidade.MPA, this.EnsaioTracao.Grafico.OpcoesGrafico.UnidadeOrdenada);
                        PontoGrafico pontoModuloSuperior = new PontoGrafico(retaModulo.obtemValorAbscissa(tensaoMaxima * 1.1), tensaoMaxima * 1.1);

                        serieModulo.Add(pontoModuloInferior);
                        serieModulo.Add(pontoModuloSuperior);
                    }
                }
            }

            this.EnsaioTracao.Grafico.SerieModuloElasticidade = serieModulo;
        }

        #endregion

        #region Serie Paralela ao Modulo

        private void redesenhaSerieParalelaModulo()
        {
            this.EnsaioTracao.Grafico.SerieParalelaModulo.Clear();
            List<PontoGrafico> serieParalela = new List<PontoGrafico>();

            if (this.EnsaioTracao.Grafico.OpcoesGrafico.ExibirParalelaModulo && this.EnsaioTracao.RetaParalelaModuloAtingida)
            {
                if (!double.IsNaN(this.EnsaioTracao.ModuloDeElasticidade) && !double.IsNaN(this.EnsaioTracao.InterseccaoOrdenadaModuloDeElasticidade) &&
                    !double.IsInfinity(this.EnsaioTracao.ModuloDeElasticidade) && !double.IsInfinity(this.EnsaioTracao.InterseccaoOrdenadaModuloDeElasticidade))
                {
                    if (this.EnsaioTracao.Grafico.OpcoesGrafico.TipoOrdenada.Equals(TipoCoordenada.TENSAO) && this.EnsaioTracao.Grafico.OpcoesGrafico.TipoAbscissa.Equals(TipoCoordenada.DEFORMACAO))
                    {
                        Extensometro extensometro = this.EnsaioTracao.Extensometro;

                        double interseccao = Conversor.converterTensao(this.EnsaioTracao.InterseccaoOrdenadaModuloDeElasticidade, Unidade.MPA, this.EnsaioTracao.Grafico.OpcoesGrafico.UnidadeOrdenada);
                        double inclinacao = Conversor.converterTensao(this.EnsaioTracao.ModuloDeElasticidade, Unidade.MPA, this.EnsaioTracao.Grafico.OpcoesGrafico.UnidadeOrdenada);
                        if (this.EnsaioTracao.Grafico.OpcoesGrafico.UnidadeAbscissa.Equals(Unidade.MM))
                        {
                            inclinacao /= extensometro.ComprimentoMedidaExtensometrico;
                        }
                        double distanciaParalela = Conversor.converterComprimento(this.EnsaioTracao.OpcoesDeEnsaio.DistanciaParalelaExtensometro, Unidade.PERCENTUAL_EXT, this.EnsaioTracao.Grafico.OpcoesGrafico.UnidadeAbscissa, extensometro.ComprimentoMedidaExtensometrico);

                        Reta retaParalela = new Reta(new Ponto(distanciaParalela, interseccao), inclinacao);

                        PontoGrafico pontoParalelaInferior = new PontoGrafico(0.0, retaParalela.obtemValorOrdenada(0.0));

                        double tensaoMaxima = Conversor.converterTensao(this.EnsaioTracao.TensaoMaxima, Unidade.MPA, this.EnsaioTracao.Grafico.OpcoesGrafico.UnidadeOrdenada);
                        PontoGrafico pontoParalelaSuperior = new PontoGrafico(retaParalela.obtemValorAbscissa(tensaoMaxima * 1.1), tensaoMaxima * 1.1);

                        serieParalela.Add(pontoParalelaInferior);
                        serieParalela.Add(pontoParalelaSuperior);
                    }
                }
            }

            this.EnsaioTracao.Grafico.SerieParalelaModulo = serieParalela;
        }

        #endregion

        #endregion

        #endregion

        #region Opcoes de grafico

        public void salvaOpcoesGrafico(OpcoesGraficoTracao opcoes)
        {
            this.EnsaioTracao.Grafico.OpcoesGrafico.salvaOpcoesGrafico(opcoes);
        }

        #endregion

        #region Opcoes de relatório

        public void salvaOpcoesRelatorio(OpcoesRelatorio opcoesRelatorio)
        {
            this.EnsaioTracao.RelatorioEnsaioTracao.OpcoesDoRelatorio.salvaOpcoesRelatorio(opcoesRelatorio);
        }

        #endregion

        #region Opcoes de ensaio

        public void salvaOpcoesEnsaio(OpcoesEnsaioTracao opcoes)
        {
            this.EnsaioTracao.OpcoesDeEnsaio.salvaOpcoesEnsaio(opcoes);
        }

        public void atualizaExtensometro(Extensometro extensometro)
        {
            this.EnsaioTracao.Extensometro.ComprimentoMedidaExtensometrico = extensometro.ComprimentoMedidaExtensometrico;
        }

        #endregion

        #region Modelo de ensaio

        public void usarEnsaioComoModelo()
        {
            ModeloDeEnsaioTracao.getInstance().usarEnsaioComoModelo(this.EnsaioTracao);
        }

        #endregion

        #region Eventos

        private void assinaListeners()
        {
            this.EnsaioTracao.Grafico.OpcoesGrafico.OpcoesDeGraficoChanged += new OpcoesGraficoTracao.OpcoesDeGraficoChangedEvent(opcoesDeGraficoChanged);
            this.EnsaioTracao.Aquisicao.ConjuntoDePontosAdicionado += new Aquisicao.ConjuntoDePontosAdicionadoEvent(conjuntoDePontosAdicionado);
            //this.EnsaioTracao.Extensometro.ComprimentoChangedChanged += new Extensometro.ComprimentoChangedEvent(extensometroComprimentoChangedChanged);
            this.EnsaioTracao.EstadoEnsaioChanged += new TestView.Ensaios.Ensaio.EstadoEnsaioChangedEvent(estadoEnsaioChanged);
        }

        private void extensometroComprimentoChangedChanged(object sender, Extensometro.ComprimentoChangedEventArgs e)
        {
            if (this.EnsaioTracao.EstadoEnsaio == EstadoEnsaio.TERMINADO)
            {
                this.rotinaPosEnsaio();
            }
        }

        private void opcoesDeGraficoChanged(object sender, OpcoesGraficoTracao.OpcoesDeGraficoChangedEventArgs e)
        {
            this.redesenhaSeries();
        }

        private void temporizador_Elapsed()
        {
            this.coletaPontos();
        }

        private void conexaoChanged(object sender, Conexao.ConexaoEventArgs e)
        {
            if (!e.EstaConectado)
            {
                this.terminaEnsaio();
            }
        }

        private void estadoEnsaioChanged(object sender, Ensaio.EstadoEnsaioEventArgs e)
        {
            if (e.EstadoEnsaio == EstadoEnsaio.TERMINADO)
            {
                this.rotinaPosEnsaio();
            }
        }

        private void conjuntoDePontosAdicionado(object sender, Aquisicao.ConjuntoDePontosAdicionadoEventArgs e)
        {
            this.analisaForcaMaxima(e.Coleta);
            this.determinaVelocidadeForca();
            this.determinaVelocidadeDeslocamento();
            this.determinarModuloElasticidade();
            this.determinaTensaoEscoamento();
            this.redesenhaSeries();
            if (this.EnsaioTracao.Extensometro != null && !this.EnsaioTracao.ExtensometroRemovido)
            {
                this.remocaoExtensometro(e.Coleta);
            }
            if (this.EnsaioTracao.OpcoesDeEnsaio.DetectarRompimento)
            {
                this.detectarRompimento(e.Coleta);
            }
        }

        #endregion
    }
}
