﻿using System;
using System.Collections.Generic;
using System.Text;
using TestView.Conexoes;
using TestView.Condicionador;
using TestView.Indicacao;
using TestView.Condicionador.ItemCalibracao;
using TestView.Serializacao;
using System.IO;
using TestView.MyMath;

namespace TestView
{
    public class Calibrador
    {
        #region Construtor e membros

        private static Calibrador _instance;
        private Conexao _conexao;
        
        public static Calibrador getInstance()
        {
            if (_instance == null)
            {
                _instance = new Calibrador();
            }
            return _instance;
        }

        private Calibrador() {}

        #endregion

        #region Conexão

        public void desconectarTudo()
        {
            foreach (Conexao conexao in GerenciadorEnsaios.getInstance().listaConexoes())
            {
                if (conexao.Conectado)
                {
                    conexao.desconectar();
                }
            }
        }

        public void conectar(Conexao conexao)
        {
            this._conexao = conexao;
            this._conexao.conectar();
            this._conexao.CondicionadorSinal.Calibracao.reiniciaCalibracao();
            this._conexao.CondicionadorSinal.carregarArquivoDeCalibracao();
            this._conexao.Indicador.iniciaIndicador();
            this._conexao.IndicacaoChanged += new Conexao.IndicacaoChangedEvent(indicacaoChanged);
            this._conexao.ConexaoChanged += new Conexao.ConexaoChangedEvent(conexaoChanged);
            this._conexao.CondicionadorSinal.Calibracao.PontosDeCalibracaoChanged += new Calibracao.PontosDeCalibracaoChangedEvent(pontosDeCalibracaoChanged);
            this._conexao.CondicionadorSinal.Calibracao.ParametrosDeCalibracaoChanged += new Calibracao.ParametrosDeCalibracaoChangedEvent(parametrosDeCalibracaoChanged);
        }

        public void desconectar()
        {
            if (this._conexao != null) 
            {
                this._conexao.Indicador.paraIndicador();
                this._conexao.desconectar();
            }
        }

        public int getBaudRate() 
        {
            return this._conexao.PortaSerial.BaudRate;
        }

        public List<Conexao> listaConexoes()
        {
            return GerenciadorEnsaios.getInstance().listaConexoes();
        }

        #endregion

        #region Eventos

        #region Conexao

        void conexaoChanged(object sender, Conexao.ConexaoEventArgs e)
        {
            this.informConexaoListeners(e.EstaConectado);
        }

        public delegate void ConexaoChangedEvent(object sender, TestView.Conexoes.Conexao.ConexaoEventArgs e);

        public event ConexaoChangedEvent ConexaoChanged;

        private void informConexaoListeners(bool estaConectado)
        {
            if (ConexaoChanged != null)
            {
                ConexaoChanged(this, new TestView.Conexoes.Conexao.ConexaoEventArgs(estaConectado));
            }
        }

        #endregion

        #region Indicacao

        #region Indicacao

        void indicacaoChanged(object sender, Indicador.IndicacaoEventArgs e)
        {
            this.informIndicacaoListeners(e.Indicacao, e.IndicacaoAjustada, e.IndicacaoAjustadaStr, e.IndicacaoRelativa);
        }

        public delegate void IndicacaoChangedEvent(object sender, TestView.Indicacao.Indicador.IndicacaoEventArgs e);

        public event IndicacaoChangedEvent IndicacaoChanged;

        private void informIndicacaoListeners(long indicacao, double indicacaoAjustada, string indicacaoAjustadaStr, double indicacaoRelativa)
        {
            if (IndicacaoChanged != null)
            {
                IndicacaoChanged(this, new TestView.Indicacao.Indicador.IndicacaoEventArgs(indicacao, indicacaoAjustada, indicacaoAjustadaStr, indicacaoRelativa));
            }
        }

        #endregion

        #endregion

        #endregion

        #region Calibracao

        public void setUnidade(Unidade unidade)
        {
            if (this._conexao != null)
            {
                this._conexao.CondicionadorSinal.Calibracao.Unidade = unidade;
            }
        }

        public Unidade getUnidade()
        {
            return this._conexao.CondicionadorSinal.Calibracao.Unidade;
        }

        public void setCapacidade(double capacidade)
        {
            if (this._conexao != null)
            {
                this._conexao.CondicionadorSinal.Calibracao.Capacidade = capacidade;
            }
        }

        public double getcapacidade()
        {
            return this._conexao.CondicionadorSinal.Calibracao.Capacidade;
        }

        public void setResolcuao(int resolucao)
        {
            if (this._conexao != null)
            {
                this._conexao.CondicionadorSinal.Calibracao.Resolucao = resolucao;
            }
        }

        public int getResolucao()
        {
            return this._conexao.CondicionadorSinal.Calibracao.Resolucao;
        }

        public void setPontoDecimal(int pontoDecimal)
        {
            if (this._conexao != null)
            {
                this._conexao.CondicionadorSinal.Calibracao.PontoDecimal = pontoDecimal;
            }
        }

        public int getPontoDecimal()
        {
            return this._conexao.CondicionadorSinal.Calibracao.PontoDecimal;
        }

        public void setTipoTransdutor(Transdutor tipo)
        {
            if (this._conexao != null)
            {
                this._conexao.CondicionadorSinal.Calibracao.TipoTransdutor = tipo;
            }
        }

        public Transdutor getTipoTransdutor()
        {
            return this._conexao.CondicionadorSinal.Calibracao.TipoTransdutor;
        }

        public void setFiltroDigital(int filtroDigital)
        {
            if (this._conexao != null)
            {
                this._conexao.setFiltroDigital(filtroDigital);
            }
        }

        public int getFiltroDigital()
        {
            return this._conexao.CondicionadorSinal.Configuracao.FiltroDigital;
        }

        public void adicionaPontoCalibracao(long ponto, double valor)
        {
            this._conexao.CondicionadorSinal.Calibracao.adicionaPontoCalibracao(ponto, valor);
        }

        public void removePontoCalibracao(int indice)
        {
            this._conexao.CondicionadorSinal.Calibracao.removePontoCalibracao(indice);
        }

        public void tara()
        {
            this._conexao.tara();
        }

        public List<PontoDeCalibracao> listaPontosDeCalibracao()
        {
            return this._conexao.CondicionadorSinal.Calibracao.PontosDeCalibracao;
        }

        public void salvarArquivoCalibracao(string nomeDoArquivo)
        {
            this._conexao.CondicionadorSinal.salvarArquivoDeCalibracao(nomeDoArquivo);
        }

        #region Eventos

        #region PontosDeCalibracaoChanged

        void pontosDeCalibracaoChanged(object sender, Calibracao.PontosDeCalibracaoEventArgs e)
        {
            this.informPontosDeCalibracaoListeners(e.PontosDeCalibracao);
        }

        public delegate void PontosDeCalibracaoChangedEvent(object sender, TestView.Condicionador.Calibracao.PontosDeCalibracaoEventArgs e);

        public event PontosDeCalibracaoChangedEvent PontosDeCalibracaoChanged;

        private void informPontosDeCalibracaoListeners(List<PontoDeCalibracao> pontosDeCalibracao)
        {
            if (PontosDeCalibracaoChanged != null)
            {
                PontosDeCalibracaoChanged(this, new TestView.Condicionador.Calibracao.PontosDeCalibracaoEventArgs(pontosDeCalibracao));
            }
        }

        #endregion

        #region ParametrosDeCalibracaoChanged

        void parametrosDeCalibracaoChanged(object sender, Calibracao.ParametrosDeCalibracaoEventArgs e)
        {
            this.informParametrosDeCalibracaoListeners(e.Tara, e.Capacidade, e.Unidade, e.TipoTransdutor, e.PontoDecimal, e.Resolucao);
        }

        public delegate void ParametrosDeCalibracaoChangedEvent(object sender, TestView.Condicionador.Calibracao.ParametrosDeCalibracaoEventArgs e);

        public event ParametrosDeCalibracaoChangedEvent ParametrosDeCalibracaoChanged;

        private void informParametrosDeCalibracaoListeners(double tara, double capacidade, Unidade unidade, Transdutor tipoTransdutor, int pontoDecimal, int resolucao)
        {
            if (ParametrosDeCalibracaoChanged != null)
            {
                ParametrosDeCalibracaoChanged(this, new TestView.Condicionador.Calibracao.ParametrosDeCalibracaoEventArgs(tara, resolucao, pontoDecimal, unidade, tipoTransdutor, capacidade));
            }
        }

        #endregion

        #endregion

        #endregion

        #region Indicacao

        public long getIndicacao()
        {
            return this._conexao.Indicador.Indicacao;
        }

        #endregion
    }
}
