﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Runtime.Serialization;
using System.IO;
using TestView.Serializacao;
using TestView.Seguranca;
using TestView.Excecoes;

namespace TestView.Condicionador
{
    [DataContract]
    public class CondicionadorSinal
    {
        #region Construtor e membros

        public const int AD104C = 0;
        public const int AD101B = 1;

        private int _codigoModelo;
        private string _arquivoCalibracao;
        private Calibracao _calibracao;
        private Configuracao _configuracao;


        public Calibracao Calibracao
        {
            get
            {
                return this._calibracao;
            }
            set
            {
                this._calibracao = value;
            }
        }

        public Configuracao Configuracao
        {
            get
            {
                return this._configuracao;
            }
            set
            {
                this._configuracao = value;
            }
        }

        public String Modelo
        {
            get
            { return dicionarioCondicionadores()[this._codigoModelo]; }
        }

        [DataMember]
        public String ArquivoCalibracao
        {
            get
            {
                return this._arquivoCalibracao;
            }
            set
            {
                this._arquivoCalibracao = value;
            }
        }

        [DataMember]
        public int CodigoModelo
        {
            get
            {
                return this._codigoModelo;
            }
            set
            {
                this._codigoModelo = value;
            }
        }

        public CondicionadorSinal(int modeloCondicionadorSinal)
        {
            this.CodigoModelo = modeloCondicionadorSinal;
            this.ArquivoCalibracao = string.Empty;
            this.Configuracao = new Configuracao();
            this.Calibracao = new Calibracao();
        }

        #endregion

        #region Arquivo de Calibracao

        public void salvarArquivoDeCalibracao(string nomeDoArquivo)
        {
            Stream stream = new FileStream(nomeDoArquivo, FileMode.Create);
            Calibracao calibracao = this.Calibracao;
            Serializador.serializaObjeto(stream, calibracao, typeof(Calibracao));
            this.ArquivoCalibracao = nomeDoArquivo;
            stream.Close();
        }

        public void salvarArquivoDeCalibracao()
        {
            this.salvarArquivoDeCalibracao(this.ArquivoCalibracao);
        }

        private void carregarArquivoDeCalibracao(string nomeDoArquivo)
        {
            if (!nomeDoArquivo.Equals(string.Empty))
            {
                if (File.Exists(nomeDoArquivo))
                {
                    try
                    {
                        Stream stream = new FileStream(nomeDoArquivo, FileMode.Open);
                        this.Calibracao = (Calibracao)Serializador.deserializaObjeto(stream, typeof(Calibracao));
                        stream.Close();
                    }
                    catch (Exception ex)
                    {
                        String mensagem = Properties.Resources.impossivelCarregarCalibracaoString;
                        mensagem = String.Format(mensagem, this.ArquivoCalibracao);
                        Logging.getInstance().error(mensagem, ex, true);
                    }
                }
                else
                {
                    String mensagem = Properties.Resources.arquivoDeCalibracaoNaoEncontradoString;
                    mensagem = String.Format(mensagem, this.ArquivoCalibracao);
                    Exception excecao = new ArquivoDeCalibracaoNaoEncontradoException(mensagem);
                    Logging.getInstance().error(mensagem, excecao, true);
                }
            }
        }

        public void carregarArquivoDeCalibracao()
        {
            this.carregarArquivoDeCalibracao(this.ArquivoCalibracao);
        }

        #endregion

        #region Serializacao

        [OnDeserializing]
        private void inicializaMembros(StreamingContext context)
        {
            this.Configuracao = new Configuracao();
            this.Calibracao = new Calibracao();
        }

        #endregion

        #region Misc

        public static Dictionary<int, String> dicionarioCondicionadores()
        {
            Dictionary<int, String> condicionadores = new Dictionary<int, String>();
            condicionadores.Add(AD104C, "AD104C");
            condicionadores.Add(AD101B, "AD101B");
            return condicionadores;
        }

        #endregion
    }
}
