﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO.Ports;
using System.Runtime.Serialization;
using TestView.Excecoes;
using TestView.Seguranca;

namespace TestView.Conexoes
{
    [DataContract]
    public class PortaSerial
    {
        #region Construtor e membros

        public const int BaudRatePadrao = 9600;
        private const int TimeoutLeituraPadrao = 1000;
        private const Parity ParidadePadrao = Parity.Even;

        private SerialPort _porta;
        private int _baudRate;
        private Parity _paridade;
        private int _timeout;
        private string _nomeDaPorta;

        [DataMember]
        public int BaudRate
        {
            get
            {
                return this._baudRate;
            }
            set
            {
                this._baudRate = value;
                this._porta.BaudRate = value;
            }
        }

        [DataMember]
        public Parity Paridade
        {
            get
            {
                return this._paridade;
            }
            set
            {
                this._paridade = value;
                this._porta.Parity = value;
            }
        }

        [DataMember]
        public int TimeoutLeitura
        {
            get
            {
                return this._timeout;
            }
            set
            {
                this._timeout = value;
                this._porta.ReadTimeout = value;
            }
        }

        [DataMember]
        public String NomePorta
        {
            get
            {
                return this._nomeDaPorta;
            }
            set
            {
                this._nomeDaPorta = value;
                this._porta.PortName = value;
            }
        }

        public PortaSerial(String nomeDaPorta)
        {
            this._porta = new SerialPort();
            this.NomePorta = nomeDaPorta;
            this.BaudRate = BaudRatePadrao;
            this.Paridade = ParidadePadrao;
            this.TimeoutLeitura = TimeoutLeituraPadrao;
        }

        public PortaSerial(String nomeDaPorta, int baudRate)
        {
            this._porta = new SerialPort();
            this.NomePorta = nomeDaPorta;
            this.BaudRate = baudRate;
            this.Paridade = ParidadePadrao;
            this.TimeoutLeitura = TimeoutLeituraPadrao;
        }

        #endregion

        #region Conexão

        public void conectar()
        {
            try
            {
                _porta.Open();
            }
            catch (Exception ex)
            {
                Logging.getInstance().error(ex.Message, ex, true);
                throw ex;
            }
        }

        public void desconectar()
        {
            try
            {
                _porta.Close();
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        public bool estaConectado()
        {
            return this._porta.IsOpen;
        }

        #endregion

        #region Dados

        public void envia(String comando)
        {
            try
            {
                this._porta.Write(comando);
            }
            catch (InvalidOperationException ex)
            {
                throw new PortaSerialDesconectadaException("A porta serial está desconectada", ex);
            }
        }

        public String recebe(int numeroBytes)
        {
            try
            {
                char[] buffer = new char[numeroBytes];
                for (int i = 0; i < numeroBytes; i++)
                {
                    buffer[i] = (char)this._porta.ReadChar();
                }
                String resposta = new String(buffer);
                return resposta;
            }
            catch (InvalidOperationException ex)
            {
                throw new PortaSerialDesconectadaException("A porta serial está desconectada", ex);
            }
            catch (System.IO.IOException ex)
            {
                throw new PortaSerialDesconectadaException("A porta serial está desconectada", ex);
            }
        }

        #endregion

        #region Serializacao

        [OnDeserializing]
        private void inicializaPortaSerial(StreamingContext context)
        {
            this._porta = new SerialPort();
        }

        #endregion

        #region Misc

        public static String[] listaPortasSeriais()
        {
            return SerialPort.GetPortNames();
        }

        public static int[] listaBaudRates()
        {
            return new int[] { 110, 300, 1200, 2400, 4800, 9600, 19200, 38400, 57600, 115200 };
        }

        #endregion
    }
}
