﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data.SqlClient;

namespace TestView.Dados
{
    public class FabricaConexoesBD
    {
        #region Construtor e membros

        private static FabricaConexoesBD _instancia;
        private List<ConexaoBD> _listaConexoes;

        private FabricaConexoesBD()
        {
            this._listaConexoes = new List<ConexaoBD>();
        }

        public static FabricaConexoesBD getInstance()
        {
            if (_instancia == null)
            {
                _instancia = new FabricaConexoesBD();
            }
            return _instancia;
        }

        #endregion

        #region Pool de conexões

        public ConexaoBD obtemConexao()
        {
            lock (this)
            {
                ConexaoBD conexao = null;
                for (int i = 0; i < this._listaConexoes.Count; i++)
                {
                    if (!this._listaConexoes[i].EmUtilizacao)
                    {
                        conexao = this._listaConexoes[i];
                    }
                }
                if (conexao == null)
                {
                    SqlConnectionStringBuilder stringBuilder = new SqlConnectionStringBuilder();
                    stringBuilder.DataSource = BancoDeDados.getInstance().DataSource;
                    stringBuilder.UserID = BancoDeDados.getInstance().NomeUsuario;
                    stringBuilder.Password = BancoDeDados.getInstance().Senha;
                    stringBuilder.InitialCatalog = BancoDeDados.getInstance().Catalog;
                    conexao = new ConexaoBD(stringBuilder.ConnectionString);
                    this._listaConexoes.Add(conexao);
                }
                conexao.EmUtilizacao = true;
                return conexao;
            }
        }

        #endregion
    }
}
