﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using TestView.Seguranca;
using System.IO;

namespace TestView.Ensaios.Modulos
{
    public class CarregadorDeModulos
    {
        #region Construtor e membros

        private static CarregadorDeModulos _instance;
        private List<BibliotecaModulo> _bibliotecasCarregadas;

        public List<BibliotecaModulo> BibliotecasCarregadas
        {
            get
            {
                return this._bibliotecasCarregadas;
            }
        }

        private CarregadorDeModulos() 
        {
            this._bibliotecasCarregadas = new List<BibliotecaModulo>();
        }

        public static CarregadorDeModulos getInstance()
        {
            if (_instance == null)
            {
                _instance = new CarregadorDeModulos();
            }
            return _instance;
        }

        #endregion

        #region Carregador

        public void carregaModulos()
        {
            this._bibliotecasCarregadas.Clear();

            /* Carrega todas as bibliotecas que possuem classes que implementam a interface IModuloDeEnsaio */
            String nomeBibliotecaPrincipal = Assembly.GetExecutingAssembly().Location;
            String diretorioBase = Path.GetDirectoryName(nomeBibliotecaPrincipal);
            String[] arquivosDll = Directory.GetFiles(diretorioBase, "*.dll", SearchOption.TopDirectoryOnly);
            foreach (String nomeDll in arquivosDll)
            {
                if (nomeDll != nomeBibliotecaPrincipal)
                {
                    try
                    {
                        Assembly assembly = Assembly.LoadFile(nomeDll);
                        Type classeModuloEnsaio = null;
                        Type classeEnsaio = null;
                        foreach (Type classe in assembly.GetTypes())
                        {
                            if (typeof(ModuloDeEnsaio).IsAssignableFrom(classe))
                            {
                                classeModuloEnsaio = classe;
                            }
                            else if (typeof(Ensaio).IsAssignableFrom(classe))
                            {
                                classeEnsaio = classe;
                            }
                        }
                        if (classeEnsaio != null && classeModuloEnsaio != null)
                        {
                            String nomeClasseModulo = classeModuloEnsaio.FullName;
                            String nomeClasseEnsaio = classeEnsaio.FullName;
                            BibliotecaModulo biblioteca = new BibliotecaModulo(assembly, nomeClasseModulo, nomeClasseEnsaio);
                            this._bibliotecasCarregadas.Add(biblioteca);
                        }
                    }
                    catch (Exception ex)
                    {
                        Logging.getInstance().debug(ex.Message, ex, false);
                    }
                }
            }

            this.informBibliotecasCarregadasListeners();
        }

        #endregion

        #region Eventos

        #region BibliotecasCarregadasChanged

        public delegate void BibliotecasCarregadasEvent(object sender, BibliotecasCarregadasEventArgs e);

        public event BibliotecasCarregadasEvent BibliotecasCarregadasChanged;

        public class BibliotecasCarregadasEventArgs : EventArgs
        {
            public BibliotecasCarregadasEventArgs(List< BibliotecaModulo> bibliotecas)
            {
                this.BibliotecasCarregadas = bibliotecas;
            }

            public List<BibliotecaModulo> BibliotecasCarregadas
            {
                get;
                set;
            }
        }

        private void informBibliotecasCarregadasListeners()
        {
            if (BibliotecasCarregadasChanged != null)
            {
                BibliotecasCarregadasChanged(this, new BibliotecasCarregadasEventArgs(this.BibliotecasCarregadas));
            }
        }

        #endregion

        #endregion
    }
}
