﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TestView.Condicionador;
using TestView.MyMath;
using TestView.Ensaios.Utilidades.Coleta;
using System.Runtime.Serialization;
using TestView.Ensaios.Utilidades.Instrumentacao;

namespace TestView.Ensaios.Utilidades
{
    [DataContract]
    public class SerieColeta
    {
        #region Construtor e membros

        private Calibracao _calibracao;
        private Instrumento _instrumento;
        private List<PontoColetado> _pontosColetados;

        [DataMember]
        public Calibracao Calibracao
        {
            get
            {
                return this._calibracao;
            }
            set
            {
                this._calibracao = value;
            }
        }
        [DataMember]
        public Instrumento Instrumento
        {
            get
            {
                return this._instrumento;
            }
            set
            {
                this._instrumento = value;
            }
        }
        [DataMember]
        public List<PontoColetado> PontosColetados
        {
            get
            {
                return this._pontosColetados;
            }
            set
            {
                this._pontosColetados = value;
            }
        }

        public SerieColeta(Calibracao calibracao, Instrumento instrumento)
        {
            this._calibracao = calibracao;
            this._instrumento = instrumento;
            this._pontosColetados = new List<PontoColetado>();
        }

        #endregion

        #region Pontos

        public void adicionaPonto(DateTime tempo, Double valor)
        {
            PontoColetado ponto = new PontoColetado(tempo, valor);
            this._pontosColetados.Add(ponto);
            this.informPontoAdicionadoListeners(tempo, valor);
        }

        #endregion

        #region Eventos

        #region PontoAdicionado

        public delegate void PontoAdicionadoEvent(object sender, PontoAdicionadoEventArgs e);

        public event PontoAdicionadoEvent PontoAdicionadoChanged;

        public class PontoAdicionadoEventArgs : EventArgs
        {
            public PontoAdicionadoEventArgs(DateTime horaColeta, Double valor)
            {
                this.HoraColeta = horaColeta;
                this.Valor = valor;
            }

            public DateTime HoraColeta
            {
                get;
                set;
            }

            public double Valor
            {
                get;
                set;
            }
        }

        private void informPontoAdicionadoListeners(DateTime horaColeta, double valor)
        {
            if (PontoAdicionadoChanged != null)
            {
                PontoAdicionadoChanged(this, new PontoAdicionadoEventArgs(horaColeta, valor));
            }
        }

        #endregion

        #endregion

        #region Serializacao

        [OnDeserializing]
        private void inicializaMembros(StreamingContext context)
        {
            this._pontosColetados = new List<PontoColetado>();
        }

        #endregion
    }
}
