﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TestView.MyMath;
using System.Runtime.Serialization;

namespace TestView.Ensaios.Utilidades.Instrumentacao
{
    [DataContract]
    [KnownType(typeof(Extensometro))]
    [KnownType(typeof(EncoderEletroMecanico))]
    [KnownType(typeof(InstrumentoNaoEspecificado))]
    public abstract class Instrumento : ICloneable
    {
        #region Membros

        private static List<Instrumento> _instrumentosDeformacao = new List<Instrumento>
        {
            new InstrumentoNaoEspecificado(),
            new Extensometro(),
        };

        private static List<Instrumento> _instrumentosDeslocamento = new List<Instrumento>
        {
            new InstrumentoNaoEspecificado(),
            new EncoderEletroMecanico(),
        };
        private static List<Instrumento> _instrumentosForca = new List<Instrumento>
        {
            new InstrumentoNaoEspecificado(),
        };
        private static List<Instrumento> _instrumentos = new List<Instrumento>
        {
            new InstrumentoNaoEspecificado(),
            new Extensometro(),
            new EncoderEletroMecanico()
        };

        public static List<Instrumento> InstrumentosDeformacao
        {
            get
            {
                return _instrumentosDeformacao;
            }
        }
        public static List<Instrumento> InstrumentosDeslocamento
        {
            get
            {
                return _instrumentosDeslocamento;
            }
        }
        public static List<Instrumento> InstrumentosForca
        {
            get
            {
                return _instrumentosForca;
            }
        }
        public static List<Instrumento> Instrumentos
        {
            get
            {
                return _instrumentos;
            }
        }

        public abstract String Descricao
        {
            get;
        }

        #endregion

        #region Misc.

        public abstract object Clone();

        public abstract override string ToString();

        #endregion
    }

    [DataContract]
    public class Extensometro : Instrumento
    {
        #region Construtor e membros

        private static string _nome = Properties.Resources.extensometroNomeString;
        private double _comprimentoMedidaExt;
        private Unidade _unidadeComprimento;

        [DataMember]
        public Double ComprimentoMedidaExtensometrico
        {
            get
            {
                return this._comprimentoMedidaExt;
            }
            set
            {
                bool informarListeners = false;
                if (value != this._comprimentoMedidaExt)
                {
                    informarListeners = true;
                }
                this._comprimentoMedidaExt = value;
                if (informarListeners)
                {
                    this.informComprimentoChangedListeners(value);
                }
            }
        }
        [DataMember]
        public Unidade UnidadeComprimento
        {
            get
            {
                return this._unidadeComprimento;
            }
            set
            {
                this._unidadeComprimento = Unidade.MapaUnidades[value.Id];
            }
        }
        public override String Descricao
        {
            get
            {
                return String.Format("{0} {1:0.0#} {2} Le", _nome, this.ComprimentoMedidaExtensometrico, this.UnidadeComprimento);
            }
        }

        public Extensometro()
        {
            this.UnidadeComprimento = Unidade.MM;
            this.ComprimentoMedidaExtensometrico = 10;
        }

        #endregion

        #region Eventos

        #region ComprimentoChanged

        public delegate void ComprimentoChangedEvent(object sender, ComprimentoChangedEventArgs e);

        public event ComprimentoChangedEvent ComprimentoChangedChanged;

        public class ComprimentoChangedEventArgs : EventArgs
        {
            public ComprimentoChangedEventArgs(double comprimento)
            {
                this.ComprimentoMedidaExtensometrico = comprimento;
            }

            public double ComprimentoMedidaExtensometrico
            {
                get;
                set;
            }
        }

        private void informComprimentoChangedListeners(Double comprimento)
        {
            if (ComprimentoChangedChanged != null)
            {
                ComprimentoChangedChanged(this, new ComprimentoChangedEventArgs(comprimento));
            }
        }

        #endregion

        #endregion

        #region Misc

        public override string ToString()
        {
            return _nome;
        }

        public override object Clone()
        {
            Extensometro extensometro = new Extensometro();
            extensometro.ComprimentoMedidaExtensometrico = this.ComprimentoMedidaExtensometrico;
            return extensometro;
        }

        #endregion
    }

    [DataContract]
    public class EncoderEletroMecanico : Instrumento
    {
        #region Construtor e membros

        private static string _nome = Properties.Resources.encoderNomeString;

        public EncoderEletroMecanico()
        {
        }

        public override String Descricao
        {
            get
            {
                return _nome;
            }
        }

        #endregion

        #region Misc

        public override string ToString()
        {
            return _nome;
        }

        public override object Clone()
        {
            return new EncoderEletroMecanico();
        }

        #endregion
    }

    [DataContract]
    public class InstrumentoNaoEspecificado : Instrumento
    {
        #region Construtor e membros

        private static string _nome = Properties.Resources.naoEspecificadoInstruentoNomeString;

        public InstrumentoNaoEspecificado()
        {
        }

        public override String Descricao
        {
            get
            {
                return _nome;
            }
        }

        #endregion

        #region Misc

        public override string ToString()
        {
            return _nome;
        }

        public override object Clone()
        {
            return new InstrumentoNaoEspecificado();
        }

        #endregion
    }
}
