﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using TestView.Condicionador.ItemCalibracao;

namespace TestView.MyMath
{
    public interface Interpolacao
    {
        #region Interpolação

        double obtemValorOrdenada(long ponto);

        #endregion
    }

    public class InterpolacaoLinear : Interpolacao 
    {
        private List<PontoDeCalibracao> _pontosDeCalibracao;

        public InterpolacaoLinear(List<PontoDeCalibracao> pontos)
        {
            this._pontosDeCalibracao = pontos;
        }

        public double obtemValorOrdenada(long ponto)
        {
            if (this._pontosDeCalibracao.Count == 0)
            {
                // Se quantidade de pontos calibrados é igual a zero,
                // não existe interpolação. Então devolvemos o valor do próprio ponto.
                return Convert.ToDouble(ponto);
            }
            else if (this._pontosDeCalibracao.Count == 1)
            {
                // Se a quantidade de pontos calibrados é igual a um,
                // devolvemos o valor do único ponto calibrado.
                return this._pontosDeCalibracao.First().Valor;
            }
            else
            {
                PontoDeCalibracao anterior = null;
                PontoDeCalibracao posterior = null;
                if (ponto < this._pontosDeCalibracao.First().Ponto)
                {
                    anterior = this._pontosDeCalibracao[0];
                    posterior = this._pontosDeCalibracao[1];
                }
                else if (ponto > this._pontosDeCalibracao.Last().Ponto)
                {
                    anterior = this._pontosDeCalibracao[this._pontosDeCalibracao.Count - 2];
                    posterior = this._pontosDeCalibracao[this._pontosDeCalibracao.Count - 1];
                }
                else
                {
                    for (int i = 1; i < this._pontosDeCalibracao.Count; i++)
                    {
                        if (ponto <= this._pontosDeCalibracao[i].Ponto)
                        {
                            anterior = this._pontosDeCalibracao[i - 1];
                            posterior = this._pontosDeCalibracao[i];
                            break;
                        }
                    }
                }
                Ponto pontoPosterior = new Ponto(posterior.Ponto, posterior.Valor);
                Ponto pontoAnterior = new Ponto(anterior.Ponto, anterior.Valor);
                Reta reta = new Reta(pontoPosterior, pontoAnterior);
                return reta.obtemValorOrdenada(ponto);
            }
        }
    }
}
