﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.Reflection;
using TestView.Ensaios;

namespace TestView.Relatorios
{
    public class RelatorioODT : ArquivoOpenDocument
    {
        private Dictionary<String, byte[]> _items;

        public RelatorioODT()
        {
            this._items = new Dictionary<String, byte[]>
            {
                {"/content.xml", this.Content},
                {"/META-INF/manifest.xml", this.Manifest},
                {"/meta.xml", this.Meta},
            };
        }

        public override Dictionary<string, byte[]> Items
        {
            get {
                return this._items;
            }
        }
        public virtual byte[] Content
        {
            get
            {
                string content = @"<?xml version=""1.0"" encoding=""UTF-8""?>

<office:document-content xmlns:office=""urn:oasis:names:tc:opendocument:xmlns:office:1.0"" xmlns:style=""urn:oasis:names:tc:opendocument:xmlns:style:1.0"" xmlns:text=""urn:oasis:names:tc:opendocument:xmlns:text:1.0"" xmlns:table=""urn:oasis:names:tc:opendocument:xmlns:table:1.0"" xmlns:draw=""urn:oasis:names:tc:opendocument:xmlns:drawing:1.0"" xmlns:fo=""urn:oasis:names:tc:opendocument:xmlns:xsl-fo-compatible:1.0"" xmlns:xlink=""http://www.w3.org/1999/xlink"" xmlns:dc=""http://purl.org/dc/elements/1.1/"" xmlns:meta=""urn:oasis:names:tc:opendocument:xmlns:meta:1.0"" xmlns:number=""urn:oasis:names:tc:opendocument:xmlns:datastyle:1.0"" xmlns:svg=""urn:oasis:names:tc:opendocument:xmlns:svg-compatible:1.0"" xmlns:chart=""urn:oasis:names:tc:opendocument:xmlns:chart:1.0"" xmlns:dr3d=""urn:oasis:names:tc:opendocument:xmlns:dr3d:1.0"" xmlns:math=""http://www.w3.org/1998/Math/MathML"" xmlns:form=""urn:oasis:names:tc:opendocument:xmlns:form:1.0"" xmlns:script=""urn:oasis:names:tc:opendocument:xmlns:script:1.0"" xmlns:ooo=""http://openoffice.org/2004/office"" xmlns:ooow=""http://openoffice.org/2004/writer"" xmlns:oooc=""http://openoffice.org/2004/calc"" xmlns:dom=""http://www.w3.org/2001/xml-events"" xmlns:xforms=""http://www.w3.org/2002/xforms"" xmlns:xsd=""http://www.w3.org/2001/XMLSchema"" xmlns:xsi=""http://www.w3.org/2001/XMLSchema-instance"" xmlns:rpt=""http://openoffice.org/2005/report"" xmlns:of=""urn:oasis:names:tc:opendocument:xmlns:of:1.2"" xmlns:xhtml=""http://www.w3.org/1999/xhtml"" xmlns:grddl=""http://www.w3.org/2003/g/data-view#"" xmlns:tableooo=""http://openoffice.org/2009/table"" xmlns:field=""urn:openoffice:names:experimental:ooo-ms-interop:xmlns:field:1.0"" office:version=""1.2"">
 <office:scripts/>
 <office:font-face-decls>
  <style:font-face style:name=""Mangal1"" svg:font-family=""Mangal""/>
  <style:font-face style:name=""Times New Roman"" svg:font-family=""&apos;Times New Roman&apos;"" style:font-family-generic=""roman"" style:font-pitch=""variable""/>
  <style:font-face style:name=""Arial"" svg:font-family=""Arial"" style:font-family-generic=""swiss"" style:font-pitch=""variable""/>
  <style:font-face style:name=""Mangal"" svg:font-family=""Mangal"" style:font-family-generic=""system"" style:font-pitch=""variable""/>
  <style:font-face style:name=""Microsoft YaHei"" svg:font-family=""&apos;Microsoft YaHei&apos;"" style:font-family-generic=""system"" style:font-pitch=""variable""/>
  <style:font-face style:name=""SimSun"" svg:font-family=""SimSun"" style:font-family-generic=""system"" style:font-pitch=""variable""/>
 </office:font-face-decls>
 <office:automatic-styles/>
 <office:body>
  <office:text>
   <text:sequence-decls>
    <text:sequence-decl text:display-outline-level=""0"" text:name=""Illustration""/>
    <text:sequence-decl text:display-outline-level=""0"" text:name=""Table""/>
    <text:sequence-decl text:display-outline-level=""0"" text:name=""Text""/>
    <text:sequence-decl text:display-outline-level=""0"" text:name=""Drawing""/>
   </text:sequence-decls>
   <text:p text:style-name=""Standard""/>
  </office:text>
 </office:body>
</office:document-content>";
                byte[] contentArray = Encoding.UTF8.GetBytes(content);
                return contentArray;
            }
        }
        public virtual byte[] Manifest
        {
            get
            {
                {
                    string manifest = @"<?xml version=""1.0"" encoding=""UTF-8""?>
<manifest:manifest xmlns:manifest=""urn:oasis:names:tc:opendocument:xmlns:manifest:1.0"" manifest:version=""1.2"">
    <manifest:file-entry manifest:media-type=""application/vnd.oasis.opendocument.text"" manifest:version=""1.2"" manifest:full-path=""/""/>
    <manifest:file-entry manifest:media-type=""text/xml"" manifest:full-path=""content.xml""/>
    <manifest:file-entry manifest:media-type=""text/xml"" manifest:full-path=""meta.xml"" />
</manifest:manifest>";
                    byte[] manifestArray = Encoding.UTF8.GetBytes(manifest);
                    return manifestArray;
                }
            }
        }
        public byte[] Meta
        {
            get
            {
                string meta = @"<?xml version=""1.0"" encoding=""UTF-8""?>

<office:document-meta xmlns:office=""urn:oasis:names:tc:opendocument:xmlns:office:1.0"" xmlns:xlink=""http://www.w3.org/1999/xlink"" xmlns:dc=""http://purl.org/dc/elements/1.1/"" xmlns:meta=""urn:oasis:names:tc:opendocument:xmlns:meta:1.0"" xmlns:ooo=""http://openoffice.org/2004/office"" xmlns:grddl=""http://www.w3.org/2003/g/data-view#"" office:version=""1.2"">
 <office:meta><meta:creation-date>{0:yyyy-MM-ddTHH:mm:ss.ff}</meta:creation-date><meta:generator>{1}</meta:generator></office:meta>
</office:document-meta>";
                String nomeAplicacao = Assembly.GetExecutingAssembly().GetName().Name;
                String versaoAplicacao = Assembly.GetExecutingAssembly().GetName().Version.ToString();
                String nomeAplicacaoGeradora = String.Format("{0} {1}", nomeAplicacao, versaoAplicacao);
                meta = String.Format(meta, DateTime.Now, nomeAplicacaoGeradora);
                byte[] metaArray = Encoding.UTF8.GetBytes(meta);
                return metaArray;
            }
        }
    }
}
