﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using TestView;
using TestView.MyMath;

namespace TestViewGUI.Calibracao
{
    public partial class FormAdicionaPonto : Form
    {
        private long _ponto;
        private Unidade _unidade;
        private String _mensagem;
        private const String mensagemErro = "{0} não pode ser convertido para um valor numérico.";
        private double _valor;

        public FormAdicionaPonto(Unidade unidade, long ponto)
        {
            InitializeComponent();
            this._unidade = unidade;
            this._ponto = ponto;
            this._valor = Double.NaN;
            this._mensagem = String.Format("Por favor, informe o valor ({0}) deste ponto:", new object[] { this._unidade });
            this.inicializaMensagem(unidade);
        }

        private void adicionaPontoCalibracao()
        {
            Calibrador.getInstance().adicionaPontoCalibracao(this._ponto, this._valor);
            this.Close();
        }

        private void inicializaMensagem(Unidade unidade)
        {
            this.labelMensagem.Text = this._mensagem;
        }

        private bool validaValor(String valorString)
        {
            try
            {
                this._valor = Double.Parse(valorString);
                return true;
            }
            catch (FormatException)
            {
                this._valor = double.NaN;
                return false;
            }
        }

        private void buttonCancelar_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void buttonOk_Click(object sender, EventArgs e)
        {
            String valor = this.textBoxValor.Text;
            if (validaValor(valor))
            {
                this.adicionaPontoCalibracao();
            }
            else
            {
                MessageBox.Show(this, String.Format(mensagemErro, valor), Application.ProductName, MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
    }
}
