﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using TestView.Conexoes;
using TestView.Condicionador;
using TestView;
using System.IO;
using System.Drawing.Imaging;
using TestView.Ensaios.Utilidades.Instrumentacao;

namespace TestViewGUI.Conexoes
{
    public partial class FormConexoes : Form
    {
        #region Construtor e membros

        private int _indice;
        private List<GroupBox> _listGroupBoxConexoes;

        public FormConexoes()
        {
            _indice = 0;
            _listGroupBoxConexoes = new List<GroupBox>();
            InitializeComponent();
        }

        #endregion

        #region Layout

        private GroupBox criaNovoGroupBoxConexao(Conexao conexao, String nomePorta, String nomeCondidionador, int baudRate)
        {
            GroupBox groupBox = new GroupBox();
            groupBox.Tag = conexao;

            groupBox.Name = "groupBoxConexao" + _indice.ToString();
            groupBox.Text = "Conexão " + (_listGroupBoxConexoes.Count + 1).ToString(); 
            //groupBox.Size = new Size(396, 62);
            groupBox.Size = new Size(this.Size.Width - 30, 62);
            groupBox.Anchor = AnchorStyles.Left | AnchorStyles.Top | AnchorStyles.Right;
            groupBox.Location = new Point(12, 41 + _listGroupBoxConexoes.Count * 68);

            PictureBox pictureBoxSerial = new PictureBox();
            pictureBoxSerial.Image = Properties.Resources.serial;
            pictureBoxSerial.Size = Properties.Resources.serial.Size;
            pictureBoxSerial.Location = new Point(3, 15);

            Label labelPorta = new Label();
            labelPorta.Name = "labelPortaSerial" + _indice.ToString();
            labelPorta.Text = "Porta Serial:";
            labelPorta.AutoSize = true;
            labelPorta.Location = new Point(50, 16);

            ComboBox comboBoxPorta = new ComboBox();
            comboBoxPorta.Name = "comboBoxPorta" + _indice.ToString();
            comboBoxPorta.Location = new Point(53, 32);
            comboBoxPorta.Size = new Size(59, 21);
            comboBoxPorta.DropDownStyle = ComboBoxStyle.DropDownList;
            comboBoxPorta.Items.AddRange(PortaSerial.listaPortasSeriais());
            if (comboBoxPorta.Items.Count > 0)
            {
                if (nomePorta != null && comboBoxPorta.Items.Contains(nomePorta))
                {
                    comboBoxPorta.SelectedItem = nomePorta;
                }
                else
                {
                    comboBoxPorta.SelectedIndex = 0;
                }
            }

            Label labelCondicionador = new Label();
            labelCondicionador.Name = "labelCondicionadorSinal" + _indice.ToString();
            labelCondicionador.Text = "Condicionador de sinal:";
            labelCondicionador.AutoSize = true;
            labelCondicionador.Location = new Point(115, 16);

            ComboBox comboBoxCondicionador = new ComboBox();
            comboBoxCondicionador.Name = "comboBoxCondicionador" + _indice.ToString();
            comboBoxCondicionador.Location = new Point(118, 32);
            comboBoxCondicionador.Size = new Size(114, 21);
            comboBoxCondicionador.DropDownStyle = ComboBoxStyle.DropDownList;
            foreach (KeyValuePair<int, String> item in CondicionadorSinal.dicionarioCondicionadores())
            {
                int idx = comboBoxCondicionador.Items.Add(item.Value);
            }
            if (comboBoxCondicionador.Items.Count > 0)
            {
                if (nomeCondidionador != null && comboBoxCondicionador.Items.Contains(nomeCondidionador))
                {
                    comboBoxCondicionador.SelectedItem = nomeCondidionador;
                }
                else
                {
                    comboBoxCondicionador.SelectedIndex = 0;
                }
            }

            Label labelBaudrate = new Label();
            labelBaudrate.Name = "labelBaudrate" + _indice.ToString();
            labelBaudrate.Text = "Baud rate:";
            labelBaudrate.AutoSize = true;
            labelBaudrate.Location = new Point(235, 16);

            ComboBox comboBoxBaudrate = new ComboBox();
            comboBoxBaudrate.Name = "comboBoxBaudrate" + _indice.ToString();
            comboBoxBaudrate.Location = new Point(238, 32);
            comboBoxBaudrate.Size = new Size(60, 21);
            comboBoxBaudrate.DropDownStyle = ComboBoxStyle.DropDownList;
            for (int i=0; i < PortaSerial.listaBaudRates().Length; i++)
            {
                comboBoxBaudrate.Items.Add(PortaSerial.listaBaudRates()[i]);
            }
            
            if (comboBoxBaudrate.Items.Count > 0)
            {
                if (comboBoxBaudrate.Items.Contains(baudRate))
                {
                    comboBoxBaudrate.SelectedItem = baudRate;
                }
                else
                {
                    comboBoxBaudrate.SelectedItem = PortaSerial.BaudRatePadrao;
                }
            }

            Button buttonRemover = new Button();
            buttonRemover.Anchor = AnchorStyles.Top | AnchorStyles.Right;
            buttonRemover.Name = "buttonRemover" + _indice.ToString();
            buttonRemover.Text = "Remover";
            buttonRemover.Location = new Point(groupBox.Size.Width - 81, 30);
            buttonRemover.Size = new Size(75, 23);
            buttonRemover.Click += new System.EventHandler(this.buttonRemover_Click);

            groupBox.Controls.Add(pictureBoxSerial);
            groupBox.Controls.Add(labelPorta);
            groupBox.Controls.Add(labelCondicionador);
            groupBox.Controls.Add(comboBoxPorta);
            groupBox.Controls.Add(comboBoxCondicionador);
            groupBox.Controls.Add(labelBaudrate);
            groupBox.Controls.Add(comboBoxBaudrate);
            groupBox.Controls.Add(buttonRemover);
            return groupBox;
        }

        private void insereGroupBoxConexao(GroupBox groupBoxConexao)
        {
            this.Controls.Add(groupBoxConexao);
            this.Size = new Size(this.Size.Width, this.Size.Height + 68);
        }

        private void adicionaNovaGroupBoxConexao(Conexao conexao, String nomePorta, String modeloCondicionadorSinal, int baudRate)
        {
            GroupBox groupBox = criaNovoGroupBoxConexao(conexao, nomePorta, modeloCondicionadorSinal, baudRate);
            _listGroupBoxConexoes.Add(groupBox);
            insereGroupBoxConexao(groupBox);
            _indice++;
        }

        private void removeGroupBoxConexao(GroupBox groupBoxConexaoRemover)
        {
            for (int i = _listGroupBoxConexoes.Count - 1; i >= 0; i--)
            {
                this.Controls.Remove(_listGroupBoxConexoes.ElementAt(i));
                this.Size = new Size(this.Size.Width, this.Size.Height - 68);
            }
            _listGroupBoxConexoes.Remove(groupBoxConexaoRemover);
            for (int i = 0; i < _listGroupBoxConexoes.Count; i++)
            {
                GroupBox groupBoxConexaoExistente = _listGroupBoxConexoes.ElementAt(i);
                groupBoxConexaoExistente.Location = new Point(12, 41 + i * 68);
                groupBoxConexaoExistente.Text = String.Format("{0} {1}", Properties.Resources.conexaoString, i + 1);
                insereGroupBoxConexao(groupBoxConexaoExistente);
            }
        }

        #endregion

        #region Conexao

        private void salvarConexoes()
        {
            List<Conexao> conexoesAtivas = new List<Conexao>();
            foreach (GroupBox groupBoxConexao in _listGroupBoxConexoes)
            {
                Conexao conexao = (Conexao)groupBoxConexao.Tag;
                String nomePorta = null;
                int modeloCondicionadorSinal = int.MaxValue;
                int baudRate = int.MaxValue;
                foreach (Control control in groupBoxConexao.Controls)
                {
                    if (control.Name.StartsWith("comboBoxPorta"))
                    {
                        ComboBox comboBoxPorta = (ComboBox)control;
                        nomePorta = comboBoxPorta.SelectedItem.ToString();
                    }
                    else if (control.Name.StartsWith("comboBoxCondicionador"))
                    {
                        ComboBox comboBoxCondicionador = (ComboBox)control;
                        modeloCondicionadorSinal = comboBoxCondicionador.SelectedIndex;
                    }
                    else if (control.Name.StartsWith("comboBoxBaudrate"))
                    {
                        ComboBox comboBoxBaudrate = (ComboBox)control;
                        baudRate = (int)comboBoxBaudrate.SelectedItem;
                    }
                }
                if (nomePorta != null && modeloCondicionadorSinal != int.MaxValue && baudRate != int.MaxValue)
                {
                    if (conexao == null)
                    {
                        conexao = GerenciadorEnsaios.getInstance().adicionaNovaConexao(nomePorta, modeloCondicionadorSinal, baudRate);
                    }
                    else
                    {
                        conexao.PortaSerial.NomePorta = nomePorta;
                        conexao.PortaSerial.BaudRate = baudRate;
                        conexao.CondicionadorSinal.CodigoModelo = modeloCondicionadorSinal;
                    }
                    conexoesAtivas.Add(conexao);
                }
            }
            List<Conexao> conexoesRemover = new List<Conexao>();
            foreach (Conexao conexao in GerenciadorEnsaios.getInstance().Conexoes)
            {
                if (!conexoesAtivas.Contains(conexao))
                {
                    conexoesRemover.Add(conexao);
                }
            }
            foreach (Conexao conexao in conexoesRemover)
            {
                GerenciadorEnsaios.getInstance().removeConexao(conexao);
            }
        }

        #endregion

        #region Eventos do form

        private void FormConexoes_Load(object sender, EventArgs e)
        {
            Calibrador.getInstance().desconectarTudo();
            foreach (Conexao conexao in GerenciadorEnsaios.getInstance().listaConexoes()) {
                adicionaNovaGroupBoxConexao(conexao, conexao.PortaSerial.NomePorta, conexao.CondicionadorSinal.Modelo, conexao.PortaSerial.BaudRate);
            }
        }

        private void buttonAdicionar_Click(object sender, EventArgs e)
        {
            adicionaNovaGroupBoxConexao(null, null, null, int.MaxValue);
        }

        private void buttonRemover_Click(object sender, EventArgs e)
        {
            Button buttonRemover = (Button)sender;
            GroupBox groupBoxConexaoRemover = (GroupBox)buttonRemover.Parent;
            removeGroupBoxConexao(groupBoxConexaoRemover);
        }

        private void buttonCancelar_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void buttonOk_Click(object sender, EventArgs e)
        {
            salvarConexoes();
            this.Close();
        }

        private void buttonAplicar_Click(object sender, EventArgs e)
        {
            salvarConexoes();
        }

        #endregion
    }
}
