﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using TestView.Seguranca;

namespace TestViewGUI.Excecoes
{
    public partial class FormAlertaExcecao : Form
    {
        private NivelLog _nivel;
        private String _mensagem;
        private Exception _excecao;

        public FormAlertaExcecao(NivelLog nivel, String mensagem, Exception excecao)
        {
            this._nivel = nivel;
            this._mensagem = mensagem;
            this._excecao = excecao;
            InitializeComponent();
            this.setLabelDetails(this.labelMensagem, this._mensagem);
            this.setFormDetails(this, String.Format("{0} - {1}", Application.ProductName, this._nivel.ToString()));
            Image imagem = null;
            if (nivel == NivelLog.DEBUG || nivel == NivelLog.INFO)
            {
                imagem = SystemIcons.Information.ToBitmap();
            }
            else if (nivel == NivelLog.ALERTA)
            {
                imagem = SystemIcons.Exclamation.ToBitmap();
            }
            else
            {
                imagem = SystemIcons.Error.ToBitmap();
            }
            this.setPictureboxDetails(this.pictureBoxIconeAlerta, imagem);
            this.setTextboxDetails(this.textBoxStacktrace, String.Format("{0}\r\n{1}", this._excecao.Message, this._excecao.StackTrace));
        }

        #region Delegates

        private delegate void SetLabelDetailsCallBack(Label label, string texto);

        private void setLabelDetails(Label label, string texto)
        {
            if (label.InvokeRequired)
            {
                SetLabelDetailsCallBack d = new SetLabelDetailsCallBack(setLabelDetails);
                try
                {
                    this.Invoke(d, new object[] { label, texto });
                }
                catch (ObjectDisposedException) { }
            }
            else
            {
                label.Text = texto;
            }
        }

        private delegate void SetFormDetailsCallBack(Form form, string titulo);

        private void setFormDetails(Form form, string titulo)
        {
            if (form.InvokeRequired)
            {
                SetFormDetailsCallBack d = new SetFormDetailsCallBack(setFormDetails);
                try
                {
                    this.Invoke(d, new object[] { form, titulo });
                }
                catch (ObjectDisposedException) { }
            }
            else
            {
                form.Text = titulo;
            }
        }

        private delegate void SetPictureboxDetailsCallBack(PictureBox pictureBox, Image imagem);

        private void setPictureboxDetails(PictureBox pictureBox, Image imagem)
        {
            if (pictureBox.InvokeRequired)
            {
                SetPictureboxDetailsCallBack d = new SetPictureboxDetailsCallBack(setPictureboxDetails);
                try
                {
                    this.Invoke(d, new object[] { pictureBox, imagem });
                }
                catch (ObjectDisposedException) { }
            }
            else
            {
                pictureBox.Image = imagem;
            }
        }

        private delegate void SetTextboxDetailsCallBack(TextBox pictureBox, String texto);

        private void setTextboxDetails(TextBox textbox, String texto)
        {
            if (textbox.InvokeRequired)
            {
                SetTextboxDetailsCallBack d = new SetTextboxDetailsCallBack(setTextboxDetails);
                try
                {
                    this.Invoke(d, new object[] { textbox, texto });
                }
                catch (ObjectDisposedException) { }
            }
            else
            {
                textbox.Text = texto;
            }
        }

        #endregion

        #region Eventos

        private void buttonFechar_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        #endregion
    }
}
