﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using TestView.Seguranca;

namespace TestViewGUI.Excexoes
{
    public partial class FormAlertaMensagem : Form
    {
        #region Construtor e membros

        private NivelLog _nivel;
        private String _mensagem;

        public FormAlertaMensagem(NivelLog nivel, String mensagem)
        {
            this._nivel = nivel;
            this._mensagem = mensagem;
            InitializeComponent();
            this.setLabelDetails(this.labelMensagem, this._mensagem.Replace(@"\n", "\n"));
            this.setFormDetails(this, String.Format("{0} - {1}", Application.ProductName, this._nivel.ToString()));
            Image imagem = null;
            if (nivel == NivelLog.DEBUG || nivel == NivelLog.INFO)
            {
                imagem = SystemIcons.Information.ToBitmap();
            }
            else if (nivel == NivelLog.ALERTA)
            {
                imagem = SystemIcons.Exclamation.ToBitmap();
            }
            else
            {
                imagem = SystemIcons.Error.ToBitmap();
            }
            this.setPictureboxDetails(this.pictureBoxIconeAlerta, imagem);
        }

        #endregion

        #region Delegates

        private delegate void SetLabelDetailsCallBack(Label label, string texto);

        private void setLabelDetails(Label label, string texto)
        {
            if (label.InvokeRequired)
            {
                SetLabelDetailsCallBack d = new SetLabelDetailsCallBack(setLabelDetails);
                try
                {
                    this.Invoke(d, new object[] { label, texto });
                }
                catch (ObjectDisposedException) { }
            }
            else
            {
                label.Text = texto;
            }
        }

        private delegate void SetFormDetailsCallBack(Form form, string titulo);

        private void setFormDetails(Form form, string titulo)
        {
            if (form.InvokeRequired)
            {
                SetFormDetailsCallBack d = new SetFormDetailsCallBack(setFormDetails);
                try
                {
                    this.Invoke(d, new object[] { form, titulo });
                }
                catch (ObjectDisposedException) { }
            }
            else
            {
                form.Text = titulo;
            }
        }

        private delegate void SetPictureboxDetailsCallBack(PictureBox pictureBox, Image imagem);

        private void setPictureboxDetails(PictureBox pictureBox, Image imagem)
        {
            if (pictureBox.InvokeRequired)
            {
                SetPictureboxDetailsCallBack d = new SetPictureboxDetailsCallBack(setPictureboxDetails);
                try
                {
                    this.Invoke(d, new object[] { pictureBox, imagem });
                }
                catch (ObjectDisposedException) { }
            }
            else
            {
                pictureBox.Image = imagem;
            }
        }

        #endregion

        #region Eventos

        private void buttonFechar_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        #endregion
    }
}
